/*****************************************************************************
 *  Rohde&Schwarz SFD DOCSIS Signal Generator instrument driver
 *  VXIpnp Instrument Driver
 *****************************************************************************/

#ifndef __RSSFD_HEADER
#define __RSSFD_HEADER

#include "rsidr_core.h"
#include "rssfd_attributes.h"
#include "rssfd_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSSFD_CLASS_DRIVER_DESCRIPTION           ""
#define RSSFD_CLASS_DRIVER_PREFIX                ""
#define RSSFD_CLASS_DRIVER_VENDOR                ""
#define RSSFD_CLASS_DRIVER_REVISION              ""
#define RSSFD_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSSFD_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSSFD_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSSFD_SPECIFIC_DRIVER_PREFIX             "RSSFD"
#define RSSFD_SPECIFIC_DRIVER_LOCATOR            ""
#define RSSFD_SPECIFIC_DRIVER_VENDOR             ""
#define RSSFD_MAJOR_VERSION                      1L    /* Instrument driver major version          */
#define RSSFD_MINOR_VERSION                      0L    /* Instrument driver minor version          */
#define RSSFD_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */          

#define RSSFD_SUPPORTED_INSTRUMENT_MODELS        ""    /* Instrument driver supported model(s)     */
#define RSSFD_GROUP_CAPABILITIES                 ""
#define RSSFD_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *------------------------------ Useful Macros -----------------------------*
 ****************************************************************************/

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/
#define RSSFD_SIMULATION_ID_QUERY      "Rohde&Schwarz,DOCSIS Signal Generator,900001,78065"  /* Default definition of ID Query Response for simulation */
#define RSSFD_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSSFD_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSSFD_IO_BUFFER_SIZE           1024L   /* I/O buffer size */
#define RSSFD_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */
#define IEEE_488_2_SRE_MASK	              0x24    /* 488.2 Event Status Register (STB) Bits */
#define IEEE_488_2_ERROR_MASK             0x4     /* 488.2 Error queue not empty bit mask */   
#define IEEE_488_2_ESE_MASK               0x39    /* 488.2 Event Status Register (ESE) Bits */

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSSFD_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSSFD_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSSFD_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSSFD_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSSFD_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSSFD_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSSFD_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSSFD_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSSFD_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSSFD_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSSFD_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSSFD_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSSFD_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSSFD_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSSFD_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */   
#define RSSFD_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSSFD_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSSFD_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSSFD_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSSFD_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSSFD_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSSFD_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSSFD_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSSFD_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSSFD_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSSFD_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSSFD_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSSFD_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSSFD_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSSFD_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSSFD_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME                                    /* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

//#define RSSFD_ATTR_ID_QUERY_RESPONSE                                     (RS_SPECIFIC_PUBLIC_ATTR_BASE + 1L)                     /* ViString (Read Only)              */

/* BEGIN GENERATE */  
#define RSSFD_ATTR_DIGITAL_TRANSMISSION_MODE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2
#define RSSFD_ATTR_OUTPUT_DOCSIS_31_PLC_MODE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11
#define RSSFD_ATTR_OUTPUT_ATTENUATION                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10
#define RSSFD_ATTR_DIGITAL_CLIPPING_MARGIN                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3
#define RSSFD_ATTR_DIGITAL_CLIPPING_STATUS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4
#define RSSFD_ATTR_USER_REFERENCE_ADJUST                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6
#define RSSFD_ATTR_USER_REFERENCE_VALUE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7
#define RSSFD_ATTR_SAVE_USER_REFERENCE_VALUE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8
#define RSSFD_ATTR_REFERENCE_OSCILLATOR_SOURCE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5
#define RSSFD_ATTR_OUTPUT_SELECT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9
#define RSSFD_ATTR_RF_TILT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 12
#define RSSFD_ATTR_DOCSIS30_BLOCK_TYPE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 13
#define RSSFD_ATTR_DOCSIS30_BLOCK_SYMBOL_RATE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 14
#define RSSFD_ATTR_DOCSIS30_BLOCK_CONSTELLATION                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 15
#define RSSFD_ATTR_DOCSIS30_BLOCK_INTERLEAVER_MODE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 16
#define RSSFD_ATTR_DOCSIS30_USEFUL_DATA_RATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 17
#define RSSFD_ATTR_DOCSIS30_TRANSMIT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 18
#define RSSFD_ATTR_DOCSIS30_SOURCE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 19
#define RSSFD_ATTR_DOCSIS30_DATA_RATE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 20
#define RSSFD_ATTR_DOCSIS30_FREQUENCY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 21
#define RSSFD_ATTR_DOCSIS30_POWER                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 22
#define RSSFD_ATTR_SYSTEM_CHANNEL_PORT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 23
#define RSSFD_ATTR_TSGEN_TRANSPORT_SELECT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 24
#define RSSFD_ATTR_TSGEN_CONTINUITY                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 25
#define RSSFD_ATTR_TSGEN_PCR                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 26
#define RSSFD_ATTR_TSGEN_TDT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 27
#define RSSFD_ATTR_DOCSIS31_TRANSMIT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 28
#define RSSFD_ATTR_DOCSIS31_CHANNEL_FREQUENCY                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 30
#define RSSFD_ATTR_DOCSIS31_FREQUENCY_SELECT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 37
#define RSSFD_ATTR_DOCSIS31_GUARD                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 38
#define RSSFD_ATTR_DOCSIS31_ESPECTRUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 44
#define RSSFD_ATTR_DOCSIS31_POWER                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 29
#define RSSFD_ATTR_DOCSIS31_FFT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 31
#define RSSFD_ATTR_DOCSIS31_WINDOWING                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 32
#define RSSFD_ATTR_DOCSIS31_CYCLIC_PREFIX                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 33
#define RSSFD_ATTR_DOCSIS31_CONT_PILOT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 34
#define RSSFD_ATTR_DOCSIS31_CONT_PILOT_PARAM                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 35
#define RSSFD_ATTR_DOCSIS31_TIME_INTERLEAVER_DEPTH                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 36
#define RSSFD_ATTR_DOCSIS31_PLC_TIMING_SYNC                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 42
#define RSSFD_ATTR_DOCSIS31_PLC_LOCATION                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 39
#define RSSFD_ATTR_DOCSIS31_PLC_SOURCE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 40
#define RSSFD_ATTR_DOCSIS31_PLC_SOURCE_PORT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 41
#define RSSFD_ATTR_DOCSIS31_NCP_ORDER                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 43
#define RSSFD_ATTR_DOCSIS31_IMPORT_ADVANCED_CONFIGURATION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 45
#define RSSFD_ATTR_DOCSIS31_EXPORT_ADVANCED_CONFIGURATION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 46
#define RSSFD_ATTR_DOCSIS31_EXCLUSION_START                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 47
#define RSSFD_ATTR_DOCSIS31_EXCLUSION_WIDTH                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 48
#define RSSFD_ATTR_DOCSIS31_PROFILE_CONSTELLATION                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 49
#define RSSFD_ATTR_DOCSIS31_PROFILE_PORT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 50
#define RSSFD_ATTR_DOCSIS31_PROFILE_SOURCE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 51
#define RSSFD_ATTR_DOCSIS31_PROFILE_FEC                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 52
#define RSSFD_ATTR_DOCSIS_31_PROFILE_CHANNEL_DATA_RATE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 53
#define RSSFD_ATTR_DOWNSTREAM_ARB_TRANSMIT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 54
#define RSSFD_ATTR_DOWNSTREAM_ARB_FREQUENCY                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 55
#define RSSFD_ATTR_DOWNSTREAM_ARB_POWER                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 56
#define RSSFD_ATTR_DOWNSTREAM_ARB_TRIGGER_MODE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 57
#define RSSFD_ATTR_DOWNSTREAM_ARB_OUTPUT_DELAY                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 58
#define RSSFD_ATTR_DOWNSTREAM_ARB_INTER_BURST_GAP                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 59
#define RSSFD_ATTR_DOWNSTREAM_ARB_WAVEFORM                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 60
#define RSSFD_ATTR_UPSTREAM_TRANSMIT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 61
#define RSSFD_ATTR_UPSTREAM_TIMING_SYNCHRONIZATION                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 62
#define RSSFD_ATTR_UPSTREAM_MODULATION_TYPE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 63
#define RSSFD_ATTR_UPSTREAM_SOURCE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 64
#define RSSFD_ATTR_UPSTREAM_CHANNEL_PORT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 71
#define RSSFD_ATTR_UPSTREAM_GRANT_SIZE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 69
#define RSSFD_ATTR_UPSTREAM_PLAYOUT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 65
#define RSSFD_ATTR_UPSTREAM_PERIODIC_START_FRAME                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 70
#define RSSFD_ATTR_UPSTREAM_PERIODIC_FRAME_REPEAT_INTERVAL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 73
#define RSSFD_ATTR_UPSTREAM_FREQUENCY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 66
#define RSSFD_ATTR_UPSTREAM_POWER                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 67
#define RSSFD_ATTR_UPSTREAM_CHANNEL_DATA_RATE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 68
#define RSSFD_ATTR_ATDMA_MINISLOT_SIZE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 74
#define RSSFD_ATTR_ATDMA_SYMBOL_RATE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 75
#define RSSFD_ATTR_ATDMA_GUARD_TIME_SIZE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 76
#define RSSFD_ATTR_ATDMA_FILL_REMAINING_MINISLOTS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 77
#define RSSFD_ATTR_ATDMA_CONSTELLATION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 78
#define RSSFD_ATTR_ATDMA_PREAMBLE_LENGTH                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 79
#define RSSFD_ATTR_ATDMA_PREAMBLE_PATTERN                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 80
#define RSSFD_ATTR_ATDMA_PREAMBLE_VALUE_OFFSET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 81
#define RSSFD_ATTR_ATDMA_PREAMBLE_TYPE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 82
#define RSSFD_ATTR_ATDMA_FORWARD_ERROR_CORRECTION                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 83
#define RSSFD_ATTR_ATDMA_FECK                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 84
#define RSSFD_ATTR_ATDMA_LAST_CODEWORD_LENGTH                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 85
#define RSSFD_ATTR_ATDMA_RS_INTERLEAVER_MODE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 86
#define RSSFD_ATTR_ATDMA_RS_INTERLEAVER_DEPTH                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 87
#define RSSFD_ATTR_ATDMA_RS_INTERLEAVER_BLOCK_SIZE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 88
#define RSSFD_ATTR_ATDMA_SCRAMBLER_STATE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 89
#define RSSFD_ATTR_ATDMA_SCRAMBLER_SEED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 90
#define RSSFD_ATTR_SCDMA_SYMBOL_RATE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 91
#define RSSFD_ATTR_SCDMA_CODES                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 92
#define RSSFD_ATTR_SCDMA_SPREADING                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 93
#define RSSFD_ATTR_SCDMA_ACTIVE_CODES                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 94
#define RSSFD_ATTR_SCDMA_MINISLOT_LOADING_DUTY_CYCLE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 95
#define RSSFD_ATTR_SCDMA_CONSTELLATION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 96
#define RSSFD_ATTR_SCDMA_PREAMBLE_PATTERN                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 97
#define RSSFD_ATTR_SCDMA_PREAMBLE_LENGTH                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 98
#define RSSFD_ATTR_SCDMA_PREAMBLE_VALUE_OFFSET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 99
#define RSSFD_ATTR_SCDMA_PREAMBLE_TYPE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 100
#define RSSFD_ATTR_SCDMA_FORWARD_ERROR_CORRECTION                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 101
#define RSSFD_ATTR_SCDMA_FECK                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 102
#define RSSFD_ATTR_SCDMA_LAST_CODEWORD_LENGTH                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 103
#define RSSFD_ATTR_SCDMA_SYMBOL_INTERLEAVER_STEP_SIZE                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 104
#define RSSFD_ATTR_SCDMA_CODE_HOPPING                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 105
#define RSSFD_ATTR_SCDMA_CODE_HOPPING_SEED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 106
#define RSSFD_ATTR_SCDMA_CODES_PER_SUBFRAME                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 107
#define RSSFD_ATTR_SCDMA_SPREADER_STATE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 108
#define RSSFD_ATTR_SCDMA_SCRAMBLER_STATE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 109
#define RSSFD_ATTR_SCDMA_SCRAMBLER_SEED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 110
#define RSSFD_ATTR_UPSTREAM_DOCSIS_TRANSMIT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 111
#define RSSFD_ATTR_UPSTREAM_DOCSIS_TIMING_SYNCHRONIZATION                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 112
#define RSSFD_ATTR_UPSTREAM_DOCSIS_BURST_TYPE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 113
#define RSSFD_ATTR_UPSTREAM_DOCSIS_SOURCE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 114
#define RSSFD_ATTR_UPSTREAM_DOCSIS_CHANNEL_PORT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 120
#define RSSFD_ATTR_UPSTREAM_DOCSIS_GRANT_SIZE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 121
#define RSSFD_ATTR_UPSTREAM_DOCSIS_PLAYOUT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 115
#define RSSFD_ATTR_UPSTREAM_DOCSIS_PERIODIC_START_FRAME                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 119
#define RSSFD_ATTR_UPSTREAM_DOCSIS_PERIODIC_FRAME_REPEAT_INTERVAL           RS_SPECIFIC_PUBLIC_ATTR_BASE + 122
#define RSSFD_ATTR_UPSTREAM_DOCSIS_FREQUENCY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 116
#define RSSFD_ATTR_UPSTREAM_DOCSIS_POWER                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 117
#define RSSFD_ATTR_UPSTREAM_DOCSIS_CHANNEL_DATA_RATE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 118
#define RSSFD_ATTR_OFDMA_FFT_SIZE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 123
#define RSSFD_ATTR_OFDMA_FIRST_ACTIVE_SUBCARRIER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 124
#define RSSFD_ATTR_OFDMA_LAST_ACTIVE_SUBCARRIER                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 125
#define RSSFD_ATTR_OFDMA_CYCLIC_PREFIX                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 126
#define RSSFD_ATTR_OFDMA_WINDOW_ROLLOFF_PERIOD                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 127
#define RSSFD_ATTR_OFDMA_SYMBOLS_PER_FRAME                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 128
#define RSSFD_ATTR_OFDMA_PILOT_PATTERN                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 129
#define RSSFD_ATTR_OFDMA_CONSTELLATION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 130
#define RSSFD_ATTR_OFDMA_SCRAMBLER_STATE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 131
#define RSSFD_ATTR_OFDMA_SCRAMBLER_SEED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 132
#define RSSFD_ATTR_OFDMA_ADVANCED_PROFILES                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 133
#define RSSFD_ATTR_OFDMA_EXCLUSION_START                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 135
#define RSSFD_ATTR_OFDMA_EXCLUSION_WIDTH                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 136
#define RSSFD_ATTR_OFDMA_MINISLOT_LOADING_DUTY_CYCLE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 134
#define RSSFD_ATTR_OFDMA_IRAN_STARTING_MINISLOT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 137
#define RSSFD_ATTR_OFDMA_IRAN_NUMBER_OF_MINISLOTS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 138
#define RSSFD_ATTR_OFDMA_IRAN_MAC_ADDRESS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 139
#define RSSFD_ATTR_OFDMA_IRAN_NUMBER_OF_SUBCARRIERS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 140
#define RSSFD_ATTR_OFDMA_IRAN_DOWNSTREAM_CHANNEL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 141
#define RSSFD_ATTR_OFDMA_IRAN_PREAMBLE_PATTERN                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 142
#define RSSFD_ATTR_OFDMA_IRAN_PREAMBLE_LENGTH                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 143
#define RSSFD_ATTR_OFDMA_IRAN_PREAMBLE_VALUE_OFFSET                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 144
#define RSSFD_ATTR_OFDMA_IRAN_MINISLOT_LOADING_DUTY_CYCLE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 145
#define RSSFD_ATTR_OFDMA_FRAN_STARTING_MINISLOT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 146
#define RSSFD_ATTR_OFDMA_FRAN_NUMBER_OF_MINISLOTS                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 147
#define RSSFD_ATTR_OFDMA_FRAN_NUMBER_OF_SUBCARRIERS                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 148
#define RSSFD_ATTR_OFDMA_FRAN_PREAMBLE_PATTERN                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 149
#define RSSFD_ATTR_OFDMA_FRAN_PREAMBLE_VALUE_OFFSET                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 150
#define RSSFD_ATTR_OFDMA_FRAN_DATA_FILENAME                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 151
#define RSSFD_ATTR_OFDMA_FRAN_MINISLOT_LOADING_DUTY_CYCLE                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 152
#define RSSFD_ATTR_OFDMA_WPR_SUBCARRIER_START                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 153
#define RSSFD_ATTR_OFDMA_WPR_SUBCARRIER_SKIP                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 154
#define RSSFD_ATTR_OFDMA_WPR_SUBCARRIER_SYMBOL_IN_FRAME                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 155
#define RSSFD_ATTR_OFDMA_WPR_STAGGER                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 156
#define RSSFD_ATTR_OFDMA_WPR_MINISLOT_LOADING_DUTY_CYCLE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 157
#define RSSFD_ATTR_OFDMA_BRM_NUMBER_OF_FRAMES                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 158
#define RSSFD_ATTR_OFDMA_BRM_MINISLOT_IN_FRAME                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 159
#define RSSFD_ATTR_OFDMA_BRM_SUBSLOT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 160
#define RSSFD_ATTR_OFDMA_BRM_NUMBER_OF_BYTES                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 161
#define RSSFD_ATTR_OFDMA_BRM_SID                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 162
#define RSSFD_ATTR_OFDMA_BRM_FILL_REMAINING_MINISLOTS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 163
#define RSSFD_ATTR_OFDMA_BRM_MINISLOT_LOADING_DUTY_CYCLE                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 164
#define RSSFD_ATTR_UPSTREAM_ARB_TRANSMIT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 165
#define RSSFD_ATTR_UPSTREAM_ARB_FREQUENCY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 166
#define RSSFD_ATTR_UPSTREAM_ARB_POWER                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 167
#define RSSFD_ATTR_UPSTREAM_ARB_TRIGGER_MODE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 168
#define RSSFD_ATTR_UPSTREAM_ARB_OUTPUT_DELAY                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 169
#define RSSFD_ATTR_UPSTREAM_ARB_INTER_BURST_GAP                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 170
#define RSSFD_ATTR_UPSTREAM_ARB_WAVEFORM                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 171
#define RSSFD_ATTR_HUM_STATE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 231
#define RSSFD_ATTR_HUM_FREQUENCY                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 232
#define RSSFD_ATTR_HUM_DEPTH                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 233
#define RSSFD_ATTR_NOISE_AWGN_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 172
#define RSSFD_ATTR_NOISE_AWGN_FREQUENCY                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 173
#define RSSFD_ATTR_NOISE_AWGN_CHANNEL                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 174
#define RSSFD_ATTR_NOISE_AWGN_BANDWIDTH                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 175
#define RSSFD_ATTR_NOISE_AWGN_AMPLITUDE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 176
#define RSSFD_ATTR_NOISE_AWGN_RECEIVER_BANDWIDTH                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 177
#define RSSFD_ATTR_NOISE_AWGN_CN_REFERENCE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 178
#define RSSFD_ATTR_BER_MODE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 179
#define RSSFD_ATTR_BER_VALUE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 180
#define RSSFD_ATTR_PHASE_NOISE_STATE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 234
#define RSSFD_ATTR_DOWNSTREAM_TIMING_DOCSIS_TIMESTAMP                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 181
#define RSSFD_ATTR_UPSTREAM_TIMING_DOCSIS_EXTENDED_TIMESTAMP                RS_SPECIFIC_PUBLIC_ATTR_BASE + 182
#define RSSFD_ATTR_UPSTREAM_TIMING_TX_TIME_OFFSET_ADJUST                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 183
#define RSSFD_ATTR_UPSTREAM_TIMING_DOCSIS_TIMESTAMP                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 184
#define RSSFD_ATTR_UPSTREAM_TIMING_MINISLOT_COUNTER                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 185
#define RSSFD_ATTR_UPSTREAM_TIMING_FRAME_COUNTER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 186
#define RSSFD_ATTR_TRIGGER_SOURCE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 187
#define RSSFD_ATTR_TRIGGER_IN_SENSITIVITY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 189
#define RSSFD_ATTR_MARKER_OUT_START_DELAY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 190
#define RSSFD_ATTR_MARKER_OUT_POLARITY                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 188
#define RSSFD_ATTR_TRIGGER_IMMEDIATE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 191
#define RSSFD_ATTR_IP_ADDRESS_COMMAND                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 192
#define RSSFD_ATTR_SUBNET_MASK                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 193
#define RSSFD_ATTR_GATEWAY                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 194
#define RSSFD_ATTR_NETWORK_TYPE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 195
#define RSSFD_ATTR_COMPUTER_NAME                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 196
#define RSSFD_ATTR_SFP_DATA_IP_ADDRESS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 197
#define RSSFD_ATTR_SFP_DATA_RATE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 198
#define RSSFD_ATTR_FILE_RECALL                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 199
#define RSSFD_ATTR_FILE_SAVE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 200
#define RSSFD_ATTR_FILE_MANAGER_EDIT_PATH                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 201
#define RSSFD_ATTR_FILE_MANAGER_DELETE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 202
#define RSSFD_ATTR_FILE_MANAGER_MAKE_DIR                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 203
#define RSSFD_ATTR_FILE_MANAGER_DELETE_DIR                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 204
#define RSSFD_ATTR_FILE_MANAGER_CATALOG                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 205
#define RSSFD_ATTR_FILE_MANAGER_CATALOG_LENGTH                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 206
#define RSSFD_ATTR_FILE_MANAGER_DIRECTORY_CATALOG                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 207
#define RSSFD_ATTR_FILE_MANAGER_DIRECTORY_CATALOG_LENGTH                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 208
#define RSSFD_ATTR_SYSTEM_ERROR                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 224
#define RSSFD_ATTR_SYSTEM_PRESET                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 225
#define RSSFD_ATTR_ID_QUERY_RESPONSE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 230

// rssfd_rngOutputSelect
#define RSSFD_VAL_OUT_SELECT_DOWN                                           0
#define RSSFD_VAL_OUT_SELECT_UP                                             1

// rssfd_rngReferenceSource
#define RSSFD_VAL_ROSC_SOURCE_INT                                           0
#define RSSFD_VAL_ROSC_SOURCE_EXT                                           1
#define RSSFD_VAL_ROSC_SOURCE_EXT2                                          2

// rssfd_rngDOCSIS31FFT
#define RSSFD_VAL_DOCSIS31_FFT_4K                                           0
#define RSSFD_VAL_DOCSIS31_FFT_8K                                           1

// rssfd_rngDOCSIS31CPilotParameter
#define RSSFD_VAL_DOCSIS31_CPILOT_PARAM_NUMBER                              0
#define RSSFD_VAL_DOCSIS31_CPILOT_PARAM_LIST                                1

// rssfd_rngDOCSIS31FrequencySelect
#define RSSFD_VAL_FREQ_SELECT_0TH                                           0
#define RSSFD_VAL_FREQ_SELECT_CENTER                                        1

// rssfd_rngDOCSIS31PLCSource
#define RSSFD_VAL_DOCSIS31_PLC_SOURCE_TEST                                  0
#define RSSFD_VAL_DOCSIS31_PLC_SOURCE_LAN                                   1

// rssfd_rngDOCSIS31NCP
#define RSSFD_VAL_DOCSIS31_NCP_QPSK                                         0
#define RSSFD_VAL_DOCSIS31_NCP_QAM16                                        1
#define RSSFD_VAL_DOCSIS31_NCP_QAM64                                        2

// rssfd_rngModulatorChannelProfileConstellation
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM16                            0
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM64                            1
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM64_128                        2
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM128                           3
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM128_256                       4
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM256                           5
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM256_512                       6
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM512                           7
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM512_1024                      8
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM1024                          9
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM1024_2048                     10
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM2048                          11
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM2048_4096                     12
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM4096                          13
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM4096_8192                     14
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM8192                          15
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM8192_16384                    16
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_QAM16384                         17
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_ZERO_LOADED                      18
#define RSSFD_VAL_DOCSIS31_PROFILE_CONSTEL_FILE                             19

// rssfd_rngDOCSIS31ProfileSource
#define RSSFD_VAL_DOCSIS31_PROFILE_SOURCE_OFF                               0
#define RSSFD_VAL_DOCSIS31_PROFILE_SOURCE_SFP                               1
#define RSSFD_VAL_DOCSIS31_PROFILE_SOURCE_LFSR                              2
#define RSSFD_VAL_DOCSIS31_PROFILE_SOURCE_FILL                              3

// rssfd_rngDOCSIS30Source
#define RSSFD_VAL_DOCSIS30_SOURCE_SFP                                       0
#define RSSFD_VAL_DOCSIS30_SOURCE_PN23                                      1
#define RSSFD_VAL_DOCSIS30_SOURCE_CW                                        2
#define RSSFD_VAL_DOCSIS30_SOURCE_MPEG                                      3

// rssfd_rngBlockConstellation
#define RSSFD_VAL_CONSTEL_QAM64                                             0
#define RSSFD_VAL_CONSTEL_QAM256                                            1

// rssfd_rngBlockType
#define RSSFD_VAL_BLOCK_TYPE_DVBC                                           0
#define RSSFD_VAL_BLOCK_TYPE_J83B                                           1
#define RSSFD_VAL_BLOCK_TYPE_J83C                                           2

// rssfd_rngUpstreamTriggerMode
#define RSSFD_VAL_TRIGGER_MODE_CONT                                         0
#define RSSFD_VAL_TRIGGER_MODE_TRIG                                         1

// rssfd_rngATDMAPreambleType
#define RSSFD_VAL_ATDMA_PREAMBLE_TYPE_QPSK0                                 0
#define RSSFD_VAL_ATDMA_PREAMBLE_TYPE_QPSK1                                 1

// rssfd_rngATDMALastCodewordLength
#define RSSFD_VAL_ATDMA_FEC_LENGTH_FIXED                                    0
#define RSSFD_VAL_ATDMA_FEC_LENGTH_SHORTENED                                1

// rssfd_rngATDMARSInterleaverMode
#define RSSFD_VAL_ATDMA_INTERLEAVER_MODE_DISABLED                           0
#define RSSFD_VAL_ATDMA_INTERLEAVER_MODE_FIXED                              1
#define RSSFD_VAL_ATDMA_INTERLEAVER_MODE_DYNAMIC                            2

// rssfd_rngSCDMAPreambleType
#define RSSFD_VAL_SCDMA_PREAMBLE_TYPE_QPSK0                                 0
#define RSSFD_VAL_SCDMA_PREAMBLE_TYPE_QPSK1                                 1

// rssfd_rngSCDMALastCodewordLength
#define RSSFD_VAL_SCDMA_FEC_LENGTH_FIXED                                    0
#define RSSFD_VAL_SCDMA_FEC_LENGTH_SHORTENED                                1

// rssfd_rngOFDMAFFTSize
#define RSSFD_VAL_OFDMA_FFT_2K                                              0
#define RSSFD_VAL_OFDMA_FFT_4K                                              1

// rssfd_rngOFDMAModulationOrder
#define RSSFD_VAL_OFDMA_MODULATION_QPSK                                     0
#define RSSFD_VAL_OFDMA_MODULATION_QAM8                                     1
#define RSSFD_VAL_OFDMA_MODULATION_QAM16                                    2
#define RSSFD_VAL_OFDMA_MODULATION_QAM32                                    3
#define RSSFD_VAL_OFDMA_MODULATION_QAM64                                    4
#define RSSFD_VAL_OFDMA_MODULATION_QAM128                                   5
#define RSSFD_VAL_OFDMA_MODULATION_QAM256                                   6
#define RSSFD_VAL_OFDMA_MODULATION_QAM512                                   7
#define RSSFD_VAL_OFDMA_MODULATION_QAM1024                                  8
#define RSSFD_VAL_OFDMA_MODULATION_QAM2048                                  9
#define RSSFD_VAL_OFDMA_MODULATION_QAM4096                                  10

// rssfd_rngNetworkType
#define RSSFD_VAL_NETWORK_TYPE_STATIC                                       0
#define RSSFD_VAL_NETWORK_TYPE_DHCP                                         1

// rssfd_rngSFPDataRate
#define RSSFD_VAL_SFP_DATA_RATE_1GB                                         0
#define RSSFD_VAL_SFP_DATA_RATE_10GB                                        1

// rssfd_rngStatusRegister
#define RSSFD_VAL_STAT_REG_QUESTIONABLE                                     1
#define RSSFD_VAL_STAT_REG_OPERATION                                        2

// rssfd_rngStatusRegisterPart
#define RSSFD_VAL_STAT_REG_COND                                             0
#define RSSFD_VAL_STAT_REG_EVENT                                            1
#define RSSFD_VAL_STAT_REG_ENABLE                                           2

// rssfd_rngTriggerSource
#define RSSFD_VAL_TRIGGER_INT                                               0
#define RSSFD_VAL_TRIGGER_EXT                                               1

// rssfd_rngTriggerLevel
#define RSSFD_VAL_TRIGGER_LEVEL_LOW                                         0
#define RSSFD_VAL_TRIGGER_LEVEL_HIGH                                        1

// rssfd_rngTriggerEdge
#define RSSFD_VAL_TRIGGER_EDGE_RISE                                         0
#define RSSFD_VAL_TRIGGER_EDGE_FALL                                         1

// rssfd_rngUpstreamSource
#define RSSFD_VAL_UPSTREAM_SOURCE_CW                                        0
#define RSSFD_VAL_UPSTREAM_SOURCE_SFP                                       1
#define RSSFD_VAL_UPSTREAM_SOURCE_PRBS                                      2

// rssfd_rngUpstreamModulationType
#define RSSFD_VAL_MODULATION_TYPE_ATDMA                                     0
#define RSSFD_VAL_MODULATION_TYPE_SCDMA                                     1

// rssfd_rngDigitalTransmissionMode
#define RSSFD_VAL_TRANSMISSION_MODE_OFF                                     0
#define RSSFD_VAL_TRANSMISSION_MODE_D31                                     1
#define RSSFD_VAL_TRANSMISSION_MODE_D30                                     2
#define RSSFD_VAL_TRANSMISSION_MODE_ARB                                     3

// rssfd_rngUpstreamDOCsisSource
#define RSSFD_VAL_UPSTREAM_DOCSIS_SOURCE_CW                                 0
#define RSSFD_VAL_UPSTREAM_DOCSIS_SOURCE_SFP                                1
#define RSSFD_VAL_UPSTREAM_DOCSIS_SOURCE_PRBS                               2
#define RSSFD_VAL_UPSTREAM_DOCSIS_SOURCE_STAT                               3

// rssfd_rngUpstreamDOCsisBurst
#define RSSFD_VAL_UPSTREAM_DOCSIS_BURST_DMOD                                0
#define RSSFD_VAL_UPSTREAM_DOCSIS_BURST_IRAN                                1
#define RSSFD_VAL_UPSTREAM_DOCSIS_BURST_FRAN                                2
#define RSSFD_VAL_UPSTREAM_DOCSIS_BURST_WBP                                 3
#define RSSFD_VAL_UPSTREAM_DOCSIS_BURST_BRM                                 4

// rssfd_rngATDMAConstellation
#define RSSFD_VAL_ATDMA_CONSTEL_QPSK                                        0
#define RSSFD_VAL_ATDMA_CONSTEL_QAM8                                        1
#define RSSFD_VAL_ATDMA_CONSTEL_QAM16                                       2
#define RSSFD_VAL_ATDMA_CONSTEL_QAM32                                       3
#define RSSFD_VAL_ATDMA_CONSTEL_QAM64                                       4
#define RSSFD_VAL_ATDMA_CONSTEL_QAM256                                      5
#define RSSFD_VAL_ATDMA_CONSTEL_DQPSK                                       6
#define RSSFD_VAL_ATDMA_CONSTEL_DQAM16                                      7

// rssfd_rngOutputPLCMode
#define RSSFD_VAL_OUT_PLCMODE_SPEC                                          0
#define RSSFD_VAL_OUT_PLCMODE_FSW                                           1

// rssfd_rngNoiseAWGNCNReference
#define RSSFD_VAL_AWGN_CN_REF_CHANNEL                                       0
#define RSSFD_VAL_AWGN_CN_REF_RECEIVER                                      1

// rssfd_rngSCDMAConstellation
#define RSSFD_VAL_SCDMA_CONSTEL_QPSK                                        0
#define RSSFD_VAL_SCDMA_CONSTEL_QAM8                                        1
#define RSSFD_VAL_SCDMA_CONSTEL_QAM16                                       2
#define RSSFD_VAL_SCDMA_CONSTEL_QAM32                                       3
#define RSSFD_VAL_SCDMA_CONSTEL_QAM64                                       4
#define RSSFD_VAL_SCDMA_CONSTEL_TCM_QPSK                                    5
#define RSSFD_VAL_SCDMA_CONSTEL_TCM_QAM8                                    6
#define RSSFD_VAL_SCDMA_CONSTEL_TCM_QAM16                                   7
#define RSSFD_VAL_SCDMA_CONSTEL_TCM_QAM32                                   8
#define RSSFD_VAL_SCDMA_CONSTEL_TCM_QAM64                                   9
#define RSSFD_VAL_SCDMA_CONSTEL_TCM_QAM128                                  10

// rssfd_rngDOCsisTimingSynchronization
#define RSSFD_VAL_DOCSIS_TSYNC_AUTO                                         0
#define RSSFD_VAL_DOCSIS_TSYNC_TRIG                                         1

// rssfd_rngStateArr
#define RSSFD_VAL_STATE_OFF                                                 0
#define RSSFD_VAL_STATE_ON                                                  1

// rssfd_rngBlockInterleaverMode
#define RSSFD_VAL_BLOCK_INTERLEVER_8_16                                     0
#define RSSFD_VAL_BLOCK_INTERLEVER_16_8                                     1
#define RSSFD_VAL_BLOCK_INTERLEVER_32_4                                     2
#define RSSFD_VAL_BLOCK_INTERLEVER_64_2                                     3
#define RSSFD_VAL_BLOCK_INTERLEVER_128_1                                    4
#define RSSFD_VAL_BLOCK_INTERLEVER_128_2                                    5
#define RSSFD_VAL_BLOCK_INTERLEVER_128_3                                    6
#define RSSFD_VAL_BLOCK_INTERLEVER_128_4                                    7
#define RSSFD_VAL_BLOCK_INTERLEVER_128_5                                    8
#define RSSFD_VAL_BLOCK_INTERLEVER_128_6                                    9
#define RSSFD_VAL_BLOCK_INTERLEVER_128_7                                    10
#define RSSFD_VAL_BLOCK_INTERLEVER_128_8                                    11
#define RSSFD_VAL_BLOCK_INTERLEVER_12_17                                    12

// rssfd_rngUpstreamPlayout
#define RSSFD_VAL_UPSTREAM_PLAYOUT_IMMEDIATE                                0
#define RSSFD_VAL_UPSTREAM_PLAYOUT_DYNAMIC                                  1
#define RSSFD_VAL_UPSTREAM_PLAYOUT_PERIODIC                                 2
#define RSSFD_VAL_UPSTREAM_PLAYOUT_TRIGGERED                                3
/* END GENERATE */
	
/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSSFD_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSSFD_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSSFD_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSSFD_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/	
/****************************************************************************
 *------------------------ Attribute Value Defines -------------------------*
 ****************************************************************************/

    /*- Instrument specific attribute value definitions --------------------*/


#define RSSFD_VAL_FILE_NEW    												0
#define RSSFD_VAL_FILE_COPY   												1
#define RSSFD_VAL_FILE_RENAME 												2
#define RSSFD_VAL_FILE_DELETE 												3
#define RSSFD_VAL_FILE_RDIR   												4
#define RSSFD_VAL_FILE_CDIR   												5

#define RSSFD_VAL_MEM_SAVE    												0
#define RSSFD_VAL_MEM_RECALL 												1


#define RSSFD_VAL_REG_PRESET 0
#define RSSFD_VAL_REG_QUEST  1
#define RSSFD_VAL_REG_OPER   2


#define RSSFD_VAL_REG_COND   0
#define RSSFD_VAL_REG_EVENT  1

#define RSSFD_VAL_REG_COND   												0
#define RSSFD_VAL_REG_EVENT  												1

#define RSSFD_VAL_FILE_OPERATION_IMPORT									0	
#define RSSFD_VAL_FILE_OPERATION_EXPORT									1

	
/****************************************************************************
 *---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/
ViStatus _VI_FUNC rssfd_init (ViRsrc resourceName, ViBoolean IDQuery,
                              ViBoolean resetDevice, ViSession* instrumentHandle);
ViStatus _VI_FUNC rssfd_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                         ViBoolean resetDevice,
                                         ViString optionString,
                                         ViSession* instrumentHandle);
ViStatus _VI_FUNC rssfd_QueryUnitMode (ViSession instrumentHandle,
                                       ViInt32* transmissionMode);
ViStatus _VI_FUNC rssfd_ConfigureRFOutput (ViSession instrumentHandle,
                                           ViInt32 PLCMode, ViReal64 attenuation);
ViStatus _VI_FUNC rssfd_ConfigureClippingMargin (ViSession instrumentHandle,
                                                 ViReal64 digitalClippingMargin);
ViStatus _VI_FUNC rssfd_QueryCurrentConfigurationDevice (ViSession instrumentHandle,
                                                         ViInt32* outputSelect);
ViStatus _VI_FUNC rssfd_ConfigureRFTilt (ViSession instrumentHandle,
                                         ViReal64 RFTilt);
ViStatus _VI_FUNC rssfd_QueryReferenceOscillatorSource (ViSession instrumentHandle,
                                                        ViInt32* referenceSource);
ViStatus _VI_FUNC rssfd_QueryUnitInformation (ViSession instrumentHandle,
                                              ViChar _VI_FAR SWVersion[],
                                              ViChar _VI_FAR FPGALogicVersion[],
                                              ViChar _VI_FAR boardPartNumber[],
                                              ViChar _VI_FAR boardPartsListRevision[],
                                              ViChar _VI_FAR serialNumber[]);
ViStatus _VI_FUNC rssfd_QueryUnitHardwareStatus (ViSession instrumentHandle,
                                                 ViChar _VI_FAR internalTemperature[],
                                                 ViChar _VI_FAR _12VPowerSupply[],
                                                 ViChar _VI_FAR _42VPowerSupply[],
                                                 ViChar _VI_FAR PLLSummary[],
                                                 ViChar _VI_FAR DLLSummary[],
                                                 ViChar _VI_FAR temperatureSummary[],
                                                 ViChar _VI_FAR fanSummary[],
                                                 ViChar _VI_FAR powerSupplySummary[]);
ViStatus _VI_FUNC rssfd_DownstreamDOCSIS30BlockConfiguration
             (ViSession instrumentHandle, ViInt32 type, ViReal64 symbolRate,
              ViInt32 constellation, ViInt32 interleaverMode);
ViStatus _VI_FUNC rssfd_QueryBlockConfiguration (ViSession instrumentHandle,
                                                 ViInt32* modulationType,
                                                 ViReal64* symbolRate,
                                                 ViInt32* constellation,
                                                 ViInt32* interleaver,
                                                 ViReal64* rollOff);
ViStatus _VI_FUNC rssfd_QueryUsefulDataRate (ViSession instrumentHandle,
                                             ViReal64* usefulDataRate);
ViStatus _VI_FUNC rssfd_DownstreamDOCSIS30ChannelConfiguration
             (ViSession instrumentHandle, ViBoolean transmit, ViInt32 source,
              ViReal64 power, ViReal64 frequency, ViInt32 port);
ViStatus _VI_FUNC rssfd_QueryChannelConfiguration (ViSession instrumentHandle,
                                                   ViBoolean* transmit,
                                                   ViInt32* source,
                                                   ViReal64* frequency,
                                                   ViReal64* power, ViInt32* port);
ViStatus _VI_FUNC rssfd_QueryDataRate (ViSession instrumentHandle,
                                       ViReal64* dataRate);
ViStatus _VI_FUNC rssfd_ConfigureMPEGTSGenerator (ViSession instrumentHandle,
                                                  ViBoolean continuityCorrection,
                                                  ViBoolean PCRPTSDTS,
                                                  ViBoolean TDTTOT,
                                                  ViString fileToPlay);
ViStatus _VI_FUNC rssfd_ConfigureDOCSIS31 (ViSession instrumentHandle,
                                           ViBoolean transmit, ViReal64 power,
                                           ViInt32 FFTSize, ViReal64 windowing,
                                           ViReal64 cyclicPrefix,
                                           ViInt32 continuousPilotParameter,
                                           ViInt32 timeInterleaverDepth,
                                           ViInt32 timingSync, ViInt32 NCPOrder);
ViStatus _VI_FUNC rssfd_ConfigureDOCSIS31CarrierDefinition
             (ViSession instrumentHandle, ViInt32 mode, ViReal64 frequency,
              ViInt32 numberOfGuardSubcarriers, ViReal64 encompassedSpectrum);
ViStatus _VI_FUNC rssfd_ConfigureDOCSIS31ContinuousPilot (ViSession instrumentHandle,
                                                          ViInt32 CPilotParameter,
                                                          ViInt32 continuousPilot,
                                                          ViInt32 listLength,
                                                          ViInt32 _VI_FAR pilotList[]);
ViStatus _VI_FUNC rssfd_ConfigureDOCSIS31PLC (ViSession instrumentHandle,
                                              ViInt32 PLCLocation,
                                              ViInt32 PLCSourcePort);
ViStatus _VI_FUNC rssfd_ConfigureDOCSIS31PLCSource (ViSession instrumentHandle,
                                                    ViInt32 PLCSource);
ViStatus _VI_FUNC rssfd_ConfigureDOCSIS31Exclusion (ViSession instrumentHandle,
                                                    ViInt32 exclusion,
                                                    ViInt32 start, ViInt32 width);
ViStatus _VI_FUNC rssfd_ConfigureDOCSIS31Profile (ViSession instrumentHandle,
                                                  ViInt32 profileID,
                                                  ViInt32 FECCodeShortening,
                                                  ViInt32 constellation,
                                                  ViInt32 profileSource,
                                                  ViInt32 port);
ViStatus _VI_FUNC rssfd_QueryDOCSIS31ProfileDataRate (ViSession instrumentHandle,
                                                      ViInt32 profile,
                                                      ViReal64* dataRate);
ViStatus _VI_FUNC rssfd_QueryDOCSIS31ChannelConfiguration
             (ViSession instrumentHandle, ViInt32 bufferSize,
              ViChar _VI_FAR configuration[]);
ViStatus _VI_FUNC rssfd_ImportExportDOCSIS31AdvancedConfiguration
             (ViSession instrumentHandle, ViInt32 fileOperation, ViString filename);
ViStatus _VI_FUNC rssfd_ConfigureDownstreamArbitraryWaveform
             (ViSession instrumentHandle, ViBoolean transmit, ViReal64 frequency,
              ViReal64 power, ViInt32 triggerMode, ViReal64 outputDelay,
              ViInt32 interBurstGap, ViString ARBFile);
ViStatus _VI_FUNC rssfd_QueryDownstreamARBConfiguration (ViSession instrumentHandle,
                                                         ViBoolean* transmit,
                                                         ViReal64* frequency,
                                                         ViReal64* power,
                                                         ViInt32* triggerMode,
                                                         ViReal64* outputDelay,
                                                         ViInt32* interBurstGap,
                                                         ViChar ARBFile[]);
ViStatus _VI_FUNC rssfd_ConfigureUpstream (ViSession instrumentHandle,
                                           ViBoolean transmit, ViInt32 timingSync,
                                           ViInt32 modulationType,
                                           ViReal64 frequency, ViReal64 power);
ViStatus _VI_FUNC rssfd_ConfigureUpstreamSourceOptions (ViSession instrumentHandle,
                                                        ViInt32 source,
                                                        ViInt32 grantSize,
                                                        ViInt32 port,
                                                        ViInt32 playout,
                                                        ViInt32 start,
                                                        ViInt32 repeat);
ViStatus _VI_FUNC rssfd_QueryUpstreamDataRate (ViSession instrumentHandle,
                                               ViReal64* dataRate);
ViStatus _VI_FUNC rssfd_QueryUpstreamChannelConfiguration
             (ViSession instrumentHandle, ViBoolean* transmit, ViInt32* timingSync,
              ViInt32* modulationType, ViInt32* source, ViInt32* port,
              ViInt32* grantSize, ViInt32* playout, ViInt32* start, ViInt32* repeat,
              ViReal64* frequency, ViReal64* power);
ViStatus _VI_FUNC rssfd_ConfigureATDMAMinislot (ViSession instrumentHandle,
                                                ViInt32 minislotSize,
                                                ViInt32 symbolRate,
                                                ViInt32 guardTimeSize,
                                                ViBoolean fillRemainingMinislots);
ViStatus _VI_FUNC rssfd_ConfigureATDMAModulation (ViSession instrumentHandle,
                                                  ViInt32 constellation);
ViStatus _VI_FUNC rssfd_ConfigureATDMAPreamble (ViSession instrumentHandle,
                                                ViString preamblePattern,
                                                ViInt32 preambleLength,
                                                ViInt32 preambleValueOffset,
                                                ViInt32 preambleType);
ViStatus _VI_FUNC rssfd_ConfigureATDMAFEC (ViSession instrumentHandle,
                                           ViInt32 FECErrorCorrection,
                                           ViInt32 FECErrCorrInformationBytes,
                                           ViInt32 lastCodewordLength,
                                           ViInt32 rSInterleaverMode,
                                           ViInt32 rSInterleaverDepth,
                                           ViInt32 rSInterleaverBlockSize);
ViStatus _VI_FUNC rssfd_ConfigureATDMAScrambler (ViSession instrumentHandle,
                                                 ViBoolean scramblerState,
                                                 ViString scramblerSeed);
ViStatus _VI_FUNC rssfd_ConfigureSCDMAMinislot (ViSession instrumentHandle,
                                                ViInt32 symbolRate,
                                                ViInt32 activeCodes,
                                                ViInt32 spreadingIntervalsPerFrame,
                                                ViInt32 codesPerMinislot,
                                                ViInt32 minislotLoadingDutyCycle);
ViStatus _VI_FUNC rssfd_ConfigureSCDMAModulation (ViSession instrumentHandle,
                                                  ViInt32 constellation);
ViStatus _VI_FUNC rssfd_ConfigureSCDMAPreamble (ViSession instrumentHandle,
                                                ViString preamblePattern,
                                                ViInt32 preambleLength,
                                                ViInt32 preambleValueOffset,
                                                ViInt32 preambleType);
ViStatus _VI_FUNC rssfd_ConfigureSCDMAFEC (ViSession instrumentHandle,
                                           ViInt32 FECErrorCorrection,
                                           ViInt32 FECErrCorrInformationBytes,
                                           ViInt32 lastCodewordLength);
ViStatus _VI_FUNC rssfd_ConfigureSCDMAFramerSpreader (ViSession instrumentHandle,
                                                      ViInt32 symbolInterleaverStepSize,
                                                      ViBoolean codeHopping,
                                                      ViBoolean spreaderState,
                                                      ViInt32 codesPerSubframe,
                                                      ViString codeHoppingSeed);
ViStatus _VI_FUNC rssfd_ConfigureSCDMAScrambler (ViSession instrumentHandle,
                                                 ViBoolean scramblerState,
                                                 ViString scramblerSeed);
ViStatus _VI_FUNC rssfd_ConfigureUpstreamDOCsis (ViSession instrumentHandle,
                                                 ViBoolean transmit,
                                                 ViInt32 syncMode,
                                                 ViInt32 burstType,
                                                 ViReal64 frequency,
                                                 ViReal64 power);
ViStatus _VI_FUNC rssfd_ConfigureUpstreamDOCsisSourceOptions
             (ViSession instrumentHandle, ViInt32 source, ViInt32 grantSize,
              ViInt32 port, ViInt32 playout, ViInt32 start, ViInt32 repeat);
ViStatus _VI_FUNC rssfd_QueryUpstreamDOCsisDataRate (ViSession instrumentHandle,
                                                     ViReal64* dataRate);
ViStatus _VI_FUNC rssfd_QueryUpstreamDOCsisChannelConfiguration
             (ViSession instrumentHandle, ViBoolean* transmit, ViInt32* syncMode,
              ViInt32* burstType, ViInt32* source, ViInt32* port,
              ViInt32* grantSize, ViInt32* playout, ViInt32* start, ViInt32* repeat,
              ViReal64* frequency, ViReal64* power);
ViStatus _VI_FUNC rssfd_ConfigureOFDMA (ViSession instrumentHandle, ViInt32 FFTSize,
                                        ViInt32 firstActiveSubcarrier,
                                        ViInt32 lastActiveSubcarrier,
                                        ViReal64 cyclicPrefix,
                                        ViReal64 windowRolloffPeriod,
                                        ViInt32 symbolsPerFrame,
                                        ViInt32 pilotPattern, ViInt32 constellation,
                                        ViBoolean scramblerState,
                                        ViString scramblerSeed);
ViStatus _VI_FUNC rssfd_ConfigureOFDMAAdvancedProfiles (ViSession instrumentHandle,
                                                        ViString advancedProfiles);
ViStatus _VI_FUNC rssfd_ConfigureOFDMAExclusionBands (ViSession instrumentHandle,
                                                      ViInt32 exclusion,
                                                      ViInt32 start, ViInt32 width);
ViStatus _VI_FUNC rssfd_ConfigureOFDMADataMode (ViSession instrumentHandle,
                                                ViInt32 minislotLoadingDutyCycle);
ViStatus _VI_FUNC rssfd_ConfigureOFDMAInitialRanging (ViSession instrumentHandle,
                                                      ViInt32 startingMinislotInFrame,
                                                      ViInt32 numberOfMinislots,
                                                      ViInt32 numberOfSubcarriers,
                                                      ViString macAddress,
                                                      ViInt32 downstreamChannelID,
                                                      ViString preamblePattern,
                                                      ViInt32 preambleLength,
                                                      ViInt32 preambleValueOffset,
                                                      ViInt32 minislotLoadingDutyCycle);
ViStatus _VI_FUNC rssfd_ConfigureOFDMAFineRanging (ViSession instrumentHandle,
                                                   ViInt32 startingMinislotInFrame,
                                                   ViInt32 numberOfMinislots,
                                                   ViInt32 numberOfSubcarriers,
                                                   ViString preamblePattern,
                                                   ViInt32 preambleValueOffset,
                                                   ViString dataFilename,
                                                   ViInt32 minislotLoadingDutyCycle);
ViStatus _VI_FUNC rssfd_ConfigureOFDMAWidebandProbe (ViSession instrumentHandle,
                                                     ViInt32 startSubcarrier,
                                                     ViInt32 subcarrierSkip,
                                                     ViInt32 symbolInFrame,
                                                     ViBoolean stagger,
                                                     ViInt32 minislotLoadingDutyCycle);
ViStatus _VI_FUNC rssfd_ConfigureOFDMABandwidthRequestedMessage
             (ViSession instrumentHandle, ViInt32 minislotInFrame, ViInt32 subslot,
              ViInt32 numberOfBytes, ViString SID,
              ViInt32 minislotLoadingDutyCycle);
ViStatus _VI_FUNC rssfd_ConfigureUpstreamArbitraryWaveform
             (ViSession instrumentHandle, ViBoolean transmit, ViReal64 frequency,
              ViReal64 power, ViInt32 triggerMode, ViReal64 outputDelay,
              ViInt32 interBurstGap, ViString ARBFile);
ViStatus _VI_FUNC rssfd_QueryUpstreamARBConfiguration (ViSession instrumentHandle,
                                                       ViBoolean* transmit,
                                                       ViReal64* frequency,
                                                       ViReal64* power,
                                                       ViInt32* triggerMode,
                                                       ViReal64* outputDelay,
                                                       ViInt32* interBurstGap,
                                                       ViChar ARBFile[]);
ViStatus _VI_FUNC rssfd_ConfigureHUM (ViSession instrumentHandle, ViBoolean state,
                                      ViReal64 frequency, ViReal64 depth);
ViStatus _VI_FUNC rssfd_ConfigureNoiseAWGN (ViSession instrumentHandle,
                                            ViBoolean AWGNMode,
                                            ViReal64 AWGNBandwidth,
                                            ViInt32 AWGNCNReference,
                                            ViReal64 AWGNAmplitude,
                                            ViReal64 AWGNReceiverBandwidth);
ViStatus _VI_FUNC rssfd_ConfigureBitErrorRate (ViSession instrumentHandle,
                                               ViBoolean BERMode,
                                               ViReal64 BERValue);
ViStatus _VI_FUNC rssfd_ConfigurePhaseNoise (ViSession instrumentHandle,
                                             ViBoolean state, ViReal64 frequency,
                                             ViReal64 level);
ViStatus _VI_FUNC rssfd_QueryPhaseNoiseLevel (ViSession instrumentHandle,
                                              ViReal64 frequency, ViReal64* level);
ViStatus _VI_FUNC rssfd_ConfigureDownstreamTimingSetting (ViSession instrumentHandle,
                                                          ViInt32 DOCSISTimestamp);
ViStatus _VI_FUNC rssfd_ConfigureUpstreamTimingSettings (ViSession instrumentHandle,
                                                         ViInt32 usedDOCSISExtendedTimestamp,
                                                         ViInt32 TXTimeOffsetAdjust,
                                                         ViInt32 DOCSISTimestamp,
                                                         ViInt32 minislotCounter,
                                                         ViInt32 frameCounter);
ViStatus _VI_FUNC rssfd_ConfigureTrigger (ViSession instrumentHandle,
                                          ViInt32 triggerSource,
                                          ViInt32 triggerInSensitivity,
                                          ViReal64 markerOutStartDelay,
                                          ViInt32 markerOutPolarity);
ViStatus _VI_FUNC rssfd_ConfigureTriggerImmediate (ViSession instrumentHandle);
ViStatus _VI_FUNC rssfd_ConfigureCommandCommunication (ViSession instrumentHandle,
                                                       ViString IPAddress,
                                                       ViString subnetMask,
                                                       ViString gateway);
ViStatus _VI_FUNC rssfd_ConfigureNetworkType (ViSession instrumentHandle,
                                              ViInt32 networkType);
ViStatus _VI_FUNC rssfd_ConfigureSFPPort (ViSession instrumentHandle,
                                          ViString IPAddress, ViInt32 dataRate);
ViStatus _VI_FUNC rssfd_FileManagerOperations (ViSession instrumentHandle,
                                               ViInt32 operation, ViString source,
                                               ViString destination);
ViStatus _VI_FUNC rssfd_FileCount (ViSession instrumentHandle,
                                   ViInt32* numberOfFiles);
ViStatus _VI_FUNC rssfd_FileDirectory (ViSession instrumentHandle,
                                       ViInt32 bufferSize, ViChar _VI_FAR output[]);
ViStatus _VI_FUNC rssfd_FileSubdirectoriesCount (ViSession instrumentHandle,
                                                 ViInt32* numberOfSubdirectories);
ViStatus _VI_FUNC rssfd_FileSubdirectories (ViSession instrumentHandle,
                                            ViInt32 bufferSize,
                                            ViChar _VI_FAR output[]);
ViStatus _VI_FUNC rssfd_SaveLoadSettings (ViSession instrumentHandle,
                                          ViInt32 operation, ViInt32 stateNumber,
                                          ViString path);
ViStatus _VI_FUNC rssfd_SetStatusRegister (ViSession instrumentHandle,
                                           ViInt32 registerOperation,
                                           ViInt32 enable);
ViStatus _VI_FUNC rssfd_GetStatusRegister (ViSession instrumentHandle,
                                           ViInt32 statusRegisterQuery,
                                           ViInt32 registerPart,
                                           ViInt32* registerValue);
ViStatus _VI_FUNC rssfd_SetAttributeViInt32 (ViSession instrumentHandle,
                                             ViString repeatedCapabilityName,
                                             ViUInt32 attributeID,
                                             ViInt32 attributeValue);
ViStatus _VI_FUNC rssfd_SetAttributeViReal64 (ViSession instrumentHandle,
                                              ViString repeatedCapabilityName,
                                              ViUInt32 attributeID,
                                              ViReal64 attributeValue);
ViStatus _VI_FUNC rssfd_SetAttributeViString (ViSession instrumentHandle,
                                              ViString repeatedCapabilityName,
                                              ViUInt32 attributeID,
                                              ViString attributeValue);
ViStatus _VI_FUNC rssfd_SetAttributeViBoolean (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViBoolean attributeValue);
ViStatus _VI_FUNC rssfd_SetAttributeViSession (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViSession attributeValue);
ViStatus _VI_FUNC rssfd_GetAttributeRepeatedCapabilityIds
             (ViSession instrumentHandle, ViUInt32 attributeID, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityId_s[]);
ViStatus _VI_FUNC rssfd_GetAttributeRepeatedCapabilityIdNames
             (ViSession instrumentHandle, ViUInt32 attributeID,
              ViString repeatedCapabilityId, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityIdName_s[]);
ViStatus _VI_FUNC rssfd_GetAttributeViInt32 (ViSession instrumentHandle,
                                             ViString repeatedCapabilityName,
                                             ViUInt32 attributeID,
                                             ViInt32* attributeValue);
ViStatus _VI_FUNC rssfd_GetAttributeViReal64 (ViSession instrumentHandle,
                                              ViString repeatedCapabilityName,
                                              ViUInt32 attributeID,
                                              ViReal64* attributeValue);
ViStatus _VI_FUNC rssfd_GetAttributeViString (ViSession instrumentHandle,
                                              ViString repeatedCapabilityName,
                                              ViUInt32 attributeID,
                                              ViInt32 bufferSize,
                                              ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rssfd_GetAttributeViBoolean (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViBoolean* attributeValue);
ViStatus _VI_FUNC rssfd_GetAttributeViSession (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViSession* attributeValue);
ViStatus _VI_FUNC rssfd_CheckAttributeViInt32 (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViInt32 attributeValue);
ViStatus _VI_FUNC rssfd_CheckAttributeViReal64 (ViSession instrumentHandle,
                                                ViString repeatedCapabilityName,
                                                ViUInt32 attributeID,
                                                ViReal64 attributeValue);
ViStatus _VI_FUNC rssfd_CheckAttributeViString (ViSession instrumentHandle,
                                                ViString repeatedCapabilityName,
                                                ViUInt32 attributeID,
                                                ViString attributeValue);
ViStatus _VI_FUNC rssfd_CheckAttributeViBoolean (ViSession instrumentHandle,
                                                 ViString repeatedCapabilityName,
                                                 ViUInt32 attributeID,
                                                 ViBoolean attributeValue);
ViStatus _VI_FUNC rssfd_CheckAttributeViSession (ViSession instrumentHandle,
                                                 ViString repeatedCapabilityName,
                                                 ViUInt32 attributeID,
                                                 ViSession attributeValue);
ViStatus _VI_FUNC rssfd_reset (ViSession instrumentHandle);
ViStatus _VI_FUNC rssfd_revision_query (ViSession instrumentHandle,
                                        ViChar _VI_FAR instrumentDriverRevision[],
                                        ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rssfd_error_query (ViSession instrumentHandle, ViInt32* errorCode,
                                     ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rssfd_error_message (ViSession instrumentHandle,
                                       ViStatus errorCode,
                                       ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rssfd_Preset (ViSession instrumentHandle);
ViStatus _VI_FUNC rssfd_ClearStatus (ViSession instrumentHandle);
ViStatus _VI_FUNC rssfd_IDQueryResponse (ViSession instrumentHandle,
                                         ViChar _VI_FAR IDQueryResponse[]);
ViStatus _VI_FUNC rssfd_ProcessAllPreviousCommands (ViSession instrumentHandle);
ViStatus _VI_FUNC rssfd_QueryOPC (ViSession instrumentHandle, ViInt32* OPC);
ViStatus _VI_FUNC rssfd_GetError (ViSession instrumentHandle, ViStatus* code,
                                  ViInt32 bufferSize, ViChar _VI_FAR description[]);
ViStatus _VI_FUNC rssfd_ClearError (ViSession instrumentHandle);
ViStatus _VI_FUNC rssfd_WriteInstrData (ViSession instrumentHandle,
                                        ViString writeBuffer);
ViStatus _VI_FUNC rssfd_ReadInstrData (ViSession instrumentHandle,
                                       ViInt32 numberOfBytesToRead,
                                       ViChar _VI_FAR readBuffer[],
                                       ViUInt32* numBytesRead);
ViStatus _VI_FUNC rssfd_ReadToFileFromInstrument (ViSession instrumentHandle,
                                                  ViString source,
                                                  ViString destination);
ViStatus _VI_FUNC rssfd_WriteFromFileToInstrument (ViSession instrumentHandle,
                                                   ViString source,
                                                   ViString destination);
ViStatus _VI_FUNC rssfd_QueryViBoolean (ViSession instrumentHandle,
                                        ViString command, ViBoolean* value);
ViStatus _VI_FUNC rssfd_QueryViInt32 (ViSession instrumentHandle, ViString command,
                                      ViInt32* value);
ViStatus _VI_FUNC rssfd_QueryViReal64 (ViSession instrumentHandle, ViString command,
                                       ViReal64* value);
ViStatus _VI_FUNC rssfd_QueryViString (ViSession instrumentHandle, ViString command,
                                       ViInt32 bufferSize, ViChar _VI_FAR value[]);
ViStatus _VI_FUNC rssfd_close (ViSession instrumentHandle);


/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSSFD_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSSFD_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSSFD_ERROR_INCORRECT_STATE        (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSSFD_ERROR_INCORRECT_RESPONSE     (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSSFD_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSSFD_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSSFD_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSSFD_ERRMSG_INCORRECT_STATE     	"Instrument in incorrect state"
#define RSSFD_ERRMSG_INCORRECT_RESPONSE    "Response incorrect; report problem to R&S support!"
#define RSSFD_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSSFD_ERROR_CODES_AND_MSGS \
    {RSSFD_WARN_MEASURE_UNCALIBRATED,  RSSFD_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSSFD_WARN_OVER_RANGE,            RSSFD_WARNMSG_OVER_RANGE},\
    {RSSFD_ERROR_INCORRECT_STATE , 	RSSFD_ERRMSG_INCORRECT_STATE },\
    {RSSFD_ERROR_INCORRECT_RESPONSE,   RSSFD_ERRMSG_INCORRECT_RESPONSE},\
    {RSSFD_ERROR_MAX_TIME_EXCEEDED,    RSSFD_ERRMSG_MAX_TIME_EXCEEDED}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSSFD_HEADER */
