/*****************************************************************************
 *  Rohde&Schwarz SFD DOCSIS Signal Generator instrument driver
 *  Original Release: July 2016
 *  By: Petr Marcalik 
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Version: 1.0.0
 *
 *  Modification History:
 *     see ChangeLog
 *
 *****************************************************************************/
#include "rssfd.h"
 

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/

static ViString sourceChannelArr[] = {"SFP","PN23","CW","ARB","MPEG",VI_NULL};
static ViString offOnArr[] = {"OFF","ON",VI_NULL};
static ViString constellationArr[] = {"QAM64","QAM256", VI_NULL};
static ViString regOperArr[] = {"PRES", "QUES", "OPER", VI_NULL};
static ViString regPartArr[] = {"COND", "EVEN", VI_NULL};

static ViString modulationTypeArr[] = {"DVBC","J83B","J83C", VI_NULL}; 
static ViString interleaverArr[] = {"(8,16)","(16,8)","(32,4)","(64,2)","(128,1)","(128,2)","(128,3)","(128,4)","(128,5)","(128,6)","(128,7)","(128,8)","(12,17)", VI_NULL};
static ViString triggerModeArr[] = {"CONT","TRIG", VI_NULL}; 
static ViString timingSyncModeArr[] = {"AUTO","TRIG", VI_NULL}; 
static ViString modulationArr[] = {"ATDM","SCDM", VI_NULL}; 
static ViString sourceArr[] = {"CW","SFP","PRBS","STAT",VI_NULL}; 
static ViString bursTypeArr[] = {"DMOD","IRAN","FRAN","WBP","BRM",VI_NULL};  
static ViString playoutArr[] = {"IMM","DYN","PER","TRIG",VI_NULL}; 

 /*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/
ViStatus rssfd_CheckStatusCallback (ViSession io);
ViStatus rssfd_WaitForOPCCallback (ViSession io);

ViStatus _VI_FUNC rssfd_RsClose (ViSession vi);

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/// HIFN  This function performs the following initialization actions:
/// HIFN     
/// HIFN     - Creates a new instrument driver session.
/// HIFN     
/// HIFN     - Opens a session to the specified device using the interface and 
/// HIFN  address you specify for the Resource Name parameter.
/// HIFN     
/// HIFN     - If the ID Query parameter is set to VI_TRUE, this function queries 
/// HIFN  the instrument ID and checks that it is valid for this instrument 
/// HIFN  driver.
/// HIFN     
/// HIFN     - If the Reset parameter is set to VI_TRUE, this function resets the 
/// HIFN  instrument to a known state.
/// HIFN     
/// HIFN     - Sends initialization commands to set the instrument to the state 
/// HIFN  necessary for the operation of the instrument driver.
/// HIFN     
/// HIFN     - Returns a ViSession handle that you use to identify the instrument 
/// HIFN  in all subsequent instrument driver function calls.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     This function creates a new session each time you invoke it. Although 
/// HIFN  you can open more than one session for the same resource, it is best 
/// HIFN  not to do so. You can use the same session in multiple program threads.
/// HIFN     
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     *IDN?
/// HIRET Returns the status code of this operation.
/// HIPAR resourceName/Pass the resource name of the device to initialize.
/// HIPAR idQuery/Specify whether you want the instrument driver to perform an ID Query.
/// HIPAR resetDevice/Specify whether you want the to reset the instrument during the 
/// HIPAR resetDevice/initialization procedure.
/// HIPAR instrSession/Returns a ViSession handle that you use to identify the instrument in 
/// HIPAR instrSession/all subsequent instrument driver function calls.
ViStatus _VI_FUNC rssfd_init(
	ViRsrc	resourceName,
	ViBoolean	idQuery,
	ViBoolean	resetDevice,
	ViSession*	newInstrSession
)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rssfd_InitWithOptions (resourceName, idQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}


/// HIFN  This function performs the following initialization actions:
/// HIFN     
/// HIFN     - Creates a new instrument driver session.
/// HIFN     
/// HIFN     - Opens a session to the specified device using the interface and 
/// HIFN  address you specify for the Resource Name parameter.
/// HIFN     
/// HIFN     - If the ID Query parameter is set to VI_TRUE, this function queries 
/// HIFN  the instrument ID and checks that it is valid for this instrument 
/// HIFN  driver.
/// HIFN     
/// HIFN     - If the Reset parameter is set to VI_TRUE, this function resets the 
/// HIFN  instrument to a known state.
/// HIFN     
/// HIFN     - Sends initialization commands to set the instrument to the state 
/// HIFN  necessary for the operation of the instrument driver.
/// HIFN     
/// HIFN     - Returns a ViSession handle that you use to identify the instrument 
/// HIFN  in all subsequent instrument driver function calls.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     This function creates a new session each time you invoke it. Although 
/// HIFN  you can open more than one session for the same resource, it is best 
/// HIFN  not to do so. You can use the same session in multiple program threads.
/// HIFN     
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     *IDN?
/// HIRET Returns the status code of this operation.
/// HIPAR resourceName/Pass the resource name of the device to initialize.
/// HIPAR idQuery/Specify whether you want the instrument driver to perform an ID Query.
/// HIPAR resetDevice/Specify whether you want the to reset the instrument during the 
/// HIPAR resetDevice/initialization procedure.
/// HIPAR optionString/You can use this control to set the initial value of certain 
/// HIPAR optionString/attributes for the session.  The following table lists the attributes 
/// HIPAR optionString/and the name you use in this parameter to identify the attribute.
/// HIPAR instrSession/Returns a ViSession handle that you use to identify the instrument in 
/// HIPAR instrSession/all subsequent instrument driver function calls.
ViStatus _VI_FUNC rssfd_InitWithOptions(
	ViRsrc	resourceName,
	ViBoolean	idQuery,
	ViBoolean	resetDevice,
	ViString	optionString,
	ViSession*	newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
 
	//-----
	   
	viCheckErr( viSetAttribute (instrSession, VI_ATTR_SUPPRESS_END_EN, VI_FALSE));;
	viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
	
	
	//-----	
	
	
	checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));

    ATTRLIST(attrList,rssfd); 
    checkErr( Rs_SpecificDriverNew (instrSession, "rssfd", newOptionString, attrList));
   
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rssfd);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));


    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));

    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rssfd_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rssfd_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (idQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSSFD_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSSFD_ATTR_ID_QUERY_RESPONSE,
                                   0, RSSFD_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSSFD_VALID_ID_RESPONSE_STRING, strlen(RSSFD_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    

    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (vi) session. */
    checkErr( rssfd_CheckStatus (instrSession));

Error:

    if (error < VI_SUCCESS)
    {
        ViSession   rmSess = VI_NULL;

        (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSess), &rmSess);
        
        if (instrSession != 0)
            (void) viClose (instrSession);
        (void) viClose (rmSess);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    if (error < VI_SUCCESS)
        (void) Rs_Dispose (*newInstrSession);

    return error;
}


/// HIFN  Configures system unit.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_TRANSMISSION_MODE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     DM:TRANsmission
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transmissionMode/Selects the transmission mode for the unit
ViStatus _VI_FUNC rssfd_QueryUnitMode(
	ViSession	instrSession,
	ViInt32*		transmissionMode
)
{
    ViStatus  error = VI_SUCCESS;
	long	  oldTimeout;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_GetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DIGITAL_TRANSMISSION_MODE,
                                               transmissionMode), 2, "Transmission Mode");

	viCheckErr( viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout ));
	
	viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 30000 ));
	
    checkErr (rssfd_CheckStatus (instrSession));

	viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout ));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

 

/// HIFN  Configures the digital clipping margin.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DIGITAL_CLIPPING_MARGIN
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     DM:CLIPping:MARGin
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR digitalClippingMargin/Sets the digital clipping margin.
ViStatus _VI_FUNC rssfd_ConfigureClippingMargin(
	ViSession	instrSession,
	ViReal64	digitalClippingMargin
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_DIGITAL_CLIPPING_MARGIN,
                                                digitalClippingMargin), 2, "Digital Clipping Margin");
    
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function queries the current configuration of the device to 
/// HIFN  determine which RF output path is
/// HIFN     being used.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_OUTPUT_SELECT
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     OUTPut:SELect?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR outputSelect/Returns the current configuration of the device to determine which RF 
/// HIPAR outputSelect/output path is being used.
ViStatus _VI_FUNC rssfd_QueryCurrentConfigurationDevice (ViSession instrSession,
                                          ViInt32* outputSelect)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rssfd_GetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_OUTPUT_SELECT,
                                               outputSelect), 2, "Output Select");
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the RF output.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_OUTPUT_DOCSIS_31_PLC_MODE
/// HIFN     RSSFD_ATTR_OUTPUT_ATTENUATION
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     OUTPut:SELect
/// HIFN     OUTPut:ATTenuation:LEVel
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR outputSelect/Select the output port.
/// HIPAR attenuation/Sets the overall power attenuation for the RF output.
ViStatus _VI_FUNC rssfd_ConfigureRFOutput(
	ViSession	instrSession,
	ViInt32		PLCMode,
	ViReal64	attenuation
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_OUTPUT_DOCSIS_31_PLC_MODE,
                                               PLCMode), 2, "PLC Mode");
	viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_OUTPUT_ATTENUATION,
                                                attenuation), 3, "Attenuation");
    
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}
/// HIFN  Adjust the tilt power difference of the entire output frequency band 
/// HIFN  at a slope of <Tilt>
/// HIFN     dB / GHz.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_RF_TILT
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     POWer:TILT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR rfTilt/This parameter allows the user to apply a digital tilt across the 
/// HIPAR rfTilt/entire spectrum to simulate real world frequency response. The value 
/// HIPAR rfTilt/can be positive or negative, a positive value means a positive slope 
/// HIPAR rfTilt/across the band
ViStatus _VI_FUNC rssfd_ConfigureRFTilt(
	ViSession	instrSession,
	ViReal64	rfTilt
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));


	viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_RF_TILT,
                                                rfTilt), 2, "RF Tilt");
    
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}



/// HIFN  This function returns source of the reference oscillator.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_REFERENCE_OSCILLATOR_SOURCE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     ROSCillator:SOURce?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR referenceSource/Returns source of the reference oscillator.
ViStatus _VI_FUNC rssfd_QueryReferenceOscillatorSource(
	ViSession	instrSession,
	ViInt32*	referenceSource
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm( rssfd_GetAttributeViInt32 (instrSession, "", 
                                                RSSFD_ATTR_REFERENCE_OSCILLATOR_SOURCE, referenceSource),
                                                2, "Reference Source");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}




ViStatus _VI_FUNC rssfd_QueryUnitHardwareStatus (ViSession instrSession,
                                                 ViChar  internalTemperature[],
                                                 ViChar  _12VPowerSupply[],
                                                 ViChar  _42VPowerSupply[],
                                                 ViChar  PLLSummary[],
                                                 ViChar  DLLSummary[],
                                                 ViChar  temperatureSummary[],
                                                 ViChar  fanSummary[],
                                                 ViChar  powerSupplySummary[]

)
{
    ViStatus  error = VI_SUCCESS;
    ViChar*     ext_buf=VI_NULL;
    ViChar*     p2buf;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (viPrintf (instrSession, "READ:UNIT?\n"));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, NULL));
   //1612M 131553-1 A 100118 72115 35.12 11.95 4.24 OK OK OK LOCKED LOCKED 
    p2buf = strtok (ext_buf, " ");
   // if (p2buf != NULL)
     //   strcpy (swVersion, p2buf);
	
    p2buf = strtok (NULL, " ");
   // if (p2buf != NULL)
   //     strcpy (boardPartNumber, p2buf);
	
    p2buf = strtok (NULL, " ");
   // if (p2buf != NULL)
   //     strcpy (boardPartsListRevision, p2buf);
	
    p2buf = strtok (NULL, " ");
   // if (p2buf != NULL)
   //     strcpy (boardSerialNumber, p2buf);
	
	p2buf = strtok (NULL, " ");
   // if (p2buf != NULL)
   //     strcpy (FPGALogicVersion, p2buf);
	
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (internalTemperature, p2buf);
	
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (_12VPowerSupply, p2buf);
	
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (_42VPowerSupply, p2buf);
	
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (temperatureSummary, p2buf);
	
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (fanSummary, p2buf);
	
	p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (powerSupplySummary, p2buf);
	
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (PLLSummary, p2buf);
	
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (DLLSummary, p2buf);
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

ViStatus _VI_FUNC rssfd_QueryUnitInformation (ViSession instrSession,
                                              ViChar  swVersion[],
                                              ViChar  FPGALogicVersion[],
                                              ViChar  boardPartNumber[],
                                              ViChar  boardPartsListRevision[],
                                              ViChar  serialNumber[])
{
    ViStatus  error = VI_SUCCESS;
    ViChar*     ext_buf=VI_NULL;
    ViChar*     p2buf;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (viPrintf (instrSession, "READ:UNIT?\n"));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, NULL));
   //1612M 131553-1 A 100118 72115 35.12 11.95 4.24 OK OK OK LOCKED LOCKED 
    p2buf = strtok (ext_buf, " ");
    if (p2buf != NULL)
        strcpy (swVersion, p2buf);
	
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (boardPartNumber, p2buf);
	
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (boardPartsListRevision, p2buf);
	
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (serialNumber, p2buf);
	
	p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (FPGALogicVersion, p2buf);
	
 /*  
 	p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (internalTemperature, p2buf);
	
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (_12VPowerSupply, p2buf);

    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (_42VPowerSupply, p2buf);

    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (temperatureSummary, p2buf);
	
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (fanSummary, p2buf);
	
	p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (powerSupplySummary, p2buf);
	
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (PLLSummary, p2buf);
	
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        strcpy (DLLSummary, p2buf);
	*/
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the DOCSIS 3.1 channel settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOCSIS31_TRANSMIT
/// HIFN     RSSFD_ATTR_DOCSIS31_POWER
/// HIFN     RSSFD_ATTR_DOCSIS31_FFT
/// HIFN     RSSFD_ATTR_DOCSIS31_WINDOWING
/// HIFN     RSSFD_ATTR_DOCSIS31_CYCLIC_PREFIX
/// HIFN     RSSFD_ATTR_DOCSIS31_CONT_PILOT
/// HIFN     RSSFD_ATTR_DOCSIS31_TIME_INTERLEAVER_DEPTH
/// HIFN     RSSFD_ATTR_DOCSIS31_PLC_TIMING_SYNCHRONIZATION
/// HIFN     RSSFD_ATTR_DOCSIS31_NCP_ORDER
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MODulator:CHANnel:DOCsis:STATe
/// HIFN     POWer:CHANnel:DOCsis:LEVel
/// HIFN     MODulator:CHANnel:FFT
/// HIFN     MODulator:CHANnel:WINdow
/// HIFN     MODulator:CHANnel:CPRefix
/// HIFN     MODulator:CHANnel:CPILot
/// HIFN     MODulator:CHANnel:INTSpan
/// HIFN     MODulator:CHANnel:DOCsis:TSYNchronization
/// HIFN     MODulator:CHANnel:NCP
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transmit/Disables or enables generation of the carrier for RF output. 
/// HIPAR power/Sets the power level for a specified channel.
/// HIPAR fftSize/FFT size of the DOCSIS 3.1 channel. This value can be set to either 4K 
/// HIPAR fftSize/or 8K.
/// HIPAR windowing/Roll-off or window of the DOCSIS 3.1 channel in us.
/// HIPAR cyclicPrefix/Cyclic Prefix of the DOCSIS 3.1 channel in us.
/// HIPAR continuousPilotDefinition/Continuous Pilot Parameter for the DOCSIS 3.1 channel.
/// HIPAR timeInterleaverDepth/Interleaver span of the channel.
/// HIPAR timingSynchronization/Specify how the PLC DOCSIS timestamp is generated. Setting this 
/// HIPAR timingSynchronization/parameter to Automatic synchronizes the timestamp as soon as the 
/// HIPAR timingSynchronization/channel is turned on. Alternatively setting this parameter to a value 
/// HIPAR timingSynchronization/of triggered causes the unit to wait for a trigger signal at which 
/// HIPAR timingSynchronization/point it will seed the DOCSIS timestamp and use that value for 
/// HIPAR timingSynchronization/injecting into the PLC messages.
/// HIPAR ncpOrder/Order of the NCP for the DOCSIS 3.1 channel.
ViStatus _VI_FUNC rssfd_ConfigureDOCSIS31(
	ViSession	instrSession,
	ViBoolean	state,
	ViReal64	power,
	ViInt32		fft,
	ViReal64	windowing,
	ViReal64	cyclicPrefix,
	ViInt32		continuousPilot,
	ViInt32		interleaverSpan,
	ViInt32		timingSync,
	ViInt32		ncp
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    
    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_DOCSIS31_TRANSMIT,
                                                 state), 2, "State");
	
    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_DOCSIS31_POWER,
                                                power), 3, "Power");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS31_FFT,
                                               fft), 4, "FFT");
	
    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_DOCSIS31_WINDOWING,
                                                windowing), 5, "Windowing");

    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_DOCSIS31_CYCLIC_PREFIX,
                                                cyclicPrefix), 6, "Cyclic Prefix");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS31_CONT_PILOT,
                                               continuousPilot), 7, "Continuous Pilot Parameter");

	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS31_TIME_INTERLEAVER_DEPTH,
                                               interleaverSpan), 8, "Interleaver Span Depth");
	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS31_PLC_TIMING_SYNC,
                                               timingSync), 9, "Timing Sync");
	
	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS31_NCP_ORDER,
                                               ncp), 10, "NCP Order");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the DOCSIS 3.1 channel continuous pilot settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOCSIS31_CONT_PILOT_PARAM
/// HIFN     RSSFD_ATTR_DOCSIS31_CONT_PILOT
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MODulator:CHANnel<Channel>:CPILot:SELect
/// HIFN     MODulator:CHANnel<Channel>:CPILot
/// HIFN     MODulator:CHANnel<Channel>:CPILot:LIST
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR CPilotParameter/Sets the pilot parameter using either a single pilot or a csv
/// HIPAR CPilotParameter/list of pilots.
/// HIPAR continuousPilot/Continuous Pilot Parameter for the DOCSIS 3.1 channel.
/// HIPAR listLength/Length of pilot list parameter.
/// HIPAR pilotList/A csv list of pilot parameters.
ViStatus _VI_FUNC rssfd_ConfigureDOCSIS31ContinuousPilot (
	ViSession instrSession,
	ViInt32 CPilotParameter,
	ViInt32 continuousPilot,
	ViInt32 listLength,
	ViInt32 pilotList[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuf = NULL;
    ViChar      *p2buf = NULL;
    ViInt32     i;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
   
    viCheckParm ( rssfd_SetAttributeViInt32 (instrSession, "", 
                RSSFD_ATTR_DOCSIS31_CONT_PILOT_PARAM, CPilotParameter),
                2, "CPilot Parameter");

    viCheckParm ( rssfd_SetAttributeViInt32 (instrSession, "", 
                RSSFD_ATTR_DOCSIS31_CONT_PILOT, continuousPilot),
                3, "Continuous Pilot");
	
    viCheckParm (listLength < 0, 5, "List Length");
    
    viCheckAlloc (pbuf = (ViChar*) malloc ((size_t)(listLength * 20 + 200)));
    p2buf = pbuf;
    
    if (listLength > 0)
    {
        pbuf += sprintf (pbuf, "MOD:CHAN:CPIL:LIST (");
        for (i=0;i< listLength;i++)
        {
            pbuf += sprintf (pbuf, "%ld,", pilotList[i]); 
        }
         
		*pbuf = '\0';    
        *--pbuf = ')';    
        viCheckErr (viPrintf (instrSession, "%s\n", p2buf));
    }     
    
    checkErr( rssfd_CheckStatus (instrSession)); 

Error:
    if (p2buf) free (p2buf);
    (void) Rs_UnlockSession(instrSession, NULL);
    return error; 
}


/// HIFN  Location of the 6 MHz band that contains the PLC in the DOCSIS 3.1 
/// HIFN  channel. Must
/// HIFN     start on a 1 MHz boundary.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOCSIS31_PLC_LOCATION
/// HIFN     RSSFD_ATTR_DOCSIS31_PLC_SOURCE
/// HIFN     RSSFD_ATTR_DOCSIS31_PLC_SOURCE_PORT
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MODulator:CHANnel<Channel>:PLCLocation
/// HIFN     MODulator:CHANnel<Channel>:PLCSource
/// HIFN     MODulator:CHANnel<Channel>:PLCPort
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR plcLocation/Subcarrier at which to start the exclusion band.
/// HIPAR plcSource/Source of the data for the PLC Sub-carriers of the DOCSIS 3.1 channel.
/// HIPAR plcSourcePort/DOCSIS 3.1 channel port for the input data if it is connected to the 
/// HIPAR plcSourcePort/SFP+1/2 ports.
ViStatus _VI_FUNC rssfd_ConfigureDOCSIS31PLC(
	ViSession	instrSession,
	ViInt32		plcLocation,
	ViInt32		plcSourcePort
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS31_PLC_LOCATION,
                                               plcLocation), 2, "PLC Location");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS31_PLC_SOURCE_PORT,
                                               plcSourcePort), 3, "PLC Source Port");
    
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Location of the 6 MHz band that contains the PLC in the DOCSIS 3.1 
/// HIFN  channel. Must
/// HIFN     start on a 1 MHz boundary.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOCSIS31_PLC_LOCATION
/// HIFN     RSSFD_ATTR_DOCSIS31_PLC_SOURCE
/// HIFN     RSSFD_ATTR_DOCSIS31_PLC_SOURCE_PORT
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MODulator:CHANnel<Channel>:PLCLocation
/// HIFN     MODulator:CHANnel<Channel>:PLCSource
/// HIFN     MODulator:CHANnel<Channel>:PLCPort
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR plcLocation/Subcarrier at which to start the exclusion band.
/// HIPAR plcSource/Source of the data for the PLC Sub-carriers of the DOCSIS 3.1 channel.
/// HIPAR plcSourcePort/DOCSIS 3.1 channel port for the input data if it is connected to the 
/// HIPAR plcSourcePort/SFP+1/2 ports.
ViStatus _VI_FUNC rssfd_ConfigureDOCSIS31PLCSource(
	ViSession	instrSession,
	ViInt32		plcSource
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));


	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS31_PLC_SOURCE,
                                               plcSource), 2, "PLC Source");

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the DOCSIS 3.1 Exclusion channel settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOCSIS31_EXCLUSION_START
/// HIFN     RSSFD_ATTR_DOCSIS31_EXCLUSION_WIDTH
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MODulator:CHANnel<Channel>:EXCLusion<Exclusion>:STaRT
/// HIFN     MODulator:CHANnel<Channel>:EXCLusion<Exclusion>:WIDTh
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/This control selects the channel.
/// HIPAR exclusion/This control selects the exclusion.
/// HIPAR start/Subcarrier at which to start the exclusion band.
/// HIPAR width/Width of an exclusion zone from the starting subcarrier.
ViStatus _VI_FUNC rssfd_ConfigureDOCSIS31Exclusion(
	ViSession	instrSession,
	ViInt32	exclusion,
	ViInt32	start,
	ViInt32	width
)
{
    ViStatus  error = VI_SUCCESS;
	ViChar    rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

   
    if (rssfd_invalidViInt32Range (exclusion, 1, 3) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Exclusion");
    }	

    viCheckParm (rssfd_GetAttributeRepCapName (instrSession,
                                                 RSSFD_ATTR_DOCSIS31_EXCLUSION_START,
                                                 "Exclusion",
                                                 exclusion - 1,
                                                 10,
                                                 rep_cap), 2, "Exclusion");    


    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSSFD_ATTR_DOCSIS31_EXCLUSION_START,
                                               start), 3, "Start");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSSFD_ATTR_DOCSIS31_EXCLUSION_WIDTH,
                                               width), 4, "Width");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the DOCSIS 3.1 Profile channel settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOCSIS31_PROFILE_FEC
/// HIFN     RSSFD_ATTR_DOCSIS31_PROFILE_CONSTELLATION
/// HIFN     RSSFD_ATTR_DOCSIS31_PROFILE_ID
/// HIFN     RSSFD_ATTR_DOCSIS31_PROFILE_SOURCE
/// HIFN     RSSFD_ATTR_DOCSIS31_PROFILE_PORT
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MODulator:CHANnel<Channel>:PROFile<Profile>:FEC
/// HIFN     MODulator:CHANnel<Channel>:PROFile<Profile>:CONStellation
/// HIFN     MODulator:CHANnel<Channel>:PROFile<Profile>:ID
/// HIFN     MODulator:CHANnel<Channel>:PROFile<Profile>:SOURce
/// HIFN     MODulator:CHANnel<Channel>:PROFile<Profile>:PORT
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR profile/This control selects the profile.
/// HIPAR fecCodeShortening/Specify the length of the FEC Codeword in bytes.
/// HIPAR constellation/
/// HIPAR profileID/ID of the profile. This is not the same as the profile number.
/// HIPAR profileSource/Source of the input data for the profile.
/// HIPAR port/Port for the input data on the profile.
ViStatus _VI_FUNC rssfd_ConfigureDOCSIS31Profile(
	ViSession	instrSession,
	ViInt32		profile,
	ViInt32		fecCodeShortening,
	ViInt32		constellation,
	ViInt32		profile_Source,
	ViInt32		port
)
{
    ViStatus  error = VI_SUCCESS;
	ViChar    rep_cap[15] = "";

    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

   
    if (rssfd_invalidViInt32Range (profile, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Profile");
    }	

   
    viCheckParm (rssfd_GetAttributeRepCapName (instrSession,
                                                 RSSFD_ATTR_DOCSIS31_PROFILE_FEC,
                                                 "Profile",
                                                 profile - 1,
                                                 10,
                                                 rep_cap), 3, "Profile");    

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSSFD_ATTR_DOCSIS31_PROFILE_FEC,
                                               fecCodeShortening), 3, "FEC Code Shortening");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSSFD_ATTR_DOCSIS31_PROFILE_CONSTELLATION,
                                               constellation), 4, "Constellation");

	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSSFD_ATTR_DOCSIS31_PROFILE_PORT,
                                               port), 6, "Port");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSSFD_ATTR_DOCSIS31_PROFILE_SOURCE,
                                               profile_Source), 5, "Profile Source");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the location of the specified frequency for DOCSIS 3.1 channels. 
/// HIFN  It can be either the 0th Subcarrier or the Center Frequency. If it 
/// HIFN  isn't specified, it is defaulted to center frequency.
/// HIFN     
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOCSIS31_FREQUENCY_SELECT
/// HIFN     RSSFD_ATTR_DOCSIS31_CHANNEL_FREQUENCY
/// HIFN     RSSFD_ATTR_DOCSIS31_GUARD
/// HIFN     RSSFD_ATTR_DOCSIS31_ESPECTRUM
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     FREQuency:CHANnel:FSELect
/// HIFN     FREQuency:CHANnel:DOCsis:FREQuency
/// HIFN     MODulator:CHANnel:GUARd
/// HIFN     MODulator:CHANnel:ESPectrum
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/Selects the carrier definition mode.
/// HIPAR frequency/Sets frequency of the 0th sub-carrier or center frequency of the 
/// HIPAR frequency/channel value.
/// HIPAR numberofGuardSubcarriers/Number of guard subcarriers on both sides of the DOCSIS 3.1 channel.
/// HIPAR encompassedSpectrum/Sets the encompassed spectrum for a specified channel.
ViStatus _VI_FUNC rssfd_ConfigureDOCSIS31CarrierDefinition(
	ViSession	instrSession,
	ViInt32		mode,	
	ViReal64	frequency,
	ViInt32		guard,
	ViReal64    encompassedSpectrum
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS31_FREQUENCY_SELECT,
                                               mode), 2, "Mode");
	
    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_DOCSIS31_CHANNEL_FREQUENCY,
                                                frequency), 3, "Frequency");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS31_GUARD,
                                               guard), 4, "Guard");
	
	viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS31_ESPECTRUM,
                                               encompassedSpectrum), 5, "Encompassed Spectrum");

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}







/// HIFN  Specify a configuration file to import or export advanced DOCSIS 3.1 channel properties.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOCSIS31_IMPORT_ADVANCED_CONFIGURATION
/// HIFN     RSSFD_ATTR_DOCSIS31_EXPORT_ADVANCED_CONFIGURATION
/// HIFN        
/// HIFN     Remote-control command(s):
/// HIFN     MODulator:CHANnel<Channel>:ADVanced
/// HIFN     MODulator:CHANnel<Channel>:ADVanced:EXPort
/// HIFN     
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileOperation/Selects the file operation.
/// HIPAR filename/Configuration file name.
ViStatus _VI_FUNC rssfd_ImportExportDOCSIS31AdvancedConfiguration (
	ViSession instrSession,
	ViInt32 fileOperation,
	ViString filename
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

     
	
	if(fileOperation == RSSFD_VAL_FILE_OPERATION_IMPORT)
	{
	    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
    	                                            RSSFD_ATTR_DOCSIS31_IMPORT_ADVANCED_CONFIGURATION,
        	                                        filename), 3, "Filename");
	}
	else
	{
	    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
    	                                            RSSFD_ATTR_DOCSIS31_EXPORT_ADVANCED_CONFIGURATION,
        	                                        filename), 3, "Filename");
	}
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}



/// HIFN  This function configures the downstream ARB channel settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOWNSTREAM_ARB_TRANSMIT
/// HIFN     RSSFD_ATTR_DOWNSTREAM_ARB_FREQUENCY
/// HIFN     RSSFD_ATTR_DOWNSTREAM_ARB_POWER
/// HIFN     RSSFD_ATTR_DOWNSTREAM_ARB_TRIGGER_MODE
/// HIFN     RSSFD_ATTR_DOWNSTREAM_ARB_OUTPUT_DELAY
/// HIFN     RSSFD_ATTR_DOWNSTREAM_ARB_INTER_BURST_GAP
/// HIFN     RSSFD_ATTR_DOWNSTREAM_ARB_WAVEFORM
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MODulator:CHANnel:ARBitrary:STATe
/// HIFN     FREQuency:CHANnel:ARBitrary:FREQuency
/// HIFN     POWer:CHANnel:ARBitrary:LEVel
/// HIFN     MODulator:CHANnel:ARBitrary:MODE
/// HIFN     MODulator:CHANnel:ARBitrary:DELay
/// HIFN     MODulator:CHANnel:ARBitrary:IBGAp
/// HIFN     MODulator:CHANnel:ARBitrary:WAVeform:SELect
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transmit/Switches the modulator on or off for a specified channel.
/// HIPAR frequency/Sets the frequency for a specified channel.
/// HIPAR power/Sets the power level for a specified channel.
/// HIPAR triggerMode/Specify whether or not the trigger is used to initiate playout of the 
/// HIPAR triggerMode/specific arb channel.
/// HIPAR outputDelay/Specify the amount of time between receiving trigger event and playing 
/// HIPAR outputDelay/out the arb waveform. Only used when the channel trigger mode is set to 
/// HIPAR outputDelay/triggered.
/// HIPAR interBurstGap/The amount of time between waveform playouts. This is only used in 
/// HIPAR interBurstGap/triggered mode. When the inter-burst gap is larger than 9999 us, the 
/// HIPAR interBurstGap/playout will only occur once, otherwise it will be periodic with the 
/// HIPAR interBurstGap/inter-burst gap.
/// HIPAR arbFile/Selects and loads an arbitrary waveform file for a upstream channel.
ViStatus _VI_FUNC rssfd_ConfigureDownstreamArbitraryWaveform(
	ViSession	instrSession,
	ViBoolean	transmit,
	ViReal64	frequency,
	ViReal64	power,
	ViInt32		triggerMode,
	ViReal64	outputDelay,
	ViInt32 	interBurstGap,
	ViString	arbFile
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    
    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_DOWNSTREAM_ARB_TRANSMIT,
                                                 transmit), 2, "Transmit");
	
    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_DOWNSTREAM_ARB_FREQUENCY,
                                                frequency), 3, "Frequency");
	    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_DOWNSTREAM_ARB_POWER,
                                                power), 4, "Power");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOWNSTREAM_ARB_TRIGGER_MODE,
                                               triggerMode), 5, "Trigger Mode");
	
    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_DOWNSTREAM_ARB_OUTPUT_DELAY,
                                                outputDelay), 6, "Output Delay");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_DOWNSTREAM_ARB_INTER_BURST_GAP,
                                                interBurstGap), 7, "Inter Burst Gap");
	
    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                               RSSFD_ATTR_DOWNSTREAM_ARB_WAVEFORM,
                                               arbFile), 8, "ARB File");

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the current parameter configuration for the 
/// HIFN  downstream arbitrary
/// HIFN     waveform generator. The query responds with a list of channel 
/// HIFN  parameter settings in the following order:
/// HIFN      STAT Output state
/// HIFN      FREQ Frequency
/// HIFN      POW Power
/// HIFN      MODE Playout mode
/// HIFN      DEL Output playout delay
/// HIFN      IBG Inter-burst gap
/// HIFN      SEL AWG file selection
/// HIFN     
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     READ:CHANnel:ARBitrary:CONFig?
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transmit/Returns the output state.
/// HIPAR frequency/Returns the frequency.
/// HIPAR power/Returns the power.
/// HIPAR triggerMode/Returns the playout mode.
/// HIPAR outputDelay/Returns the output playout delay.
/// HIPAR interBurstGap/Returns the inter-burst gap.
/// HIPAR arbFile/Returns the AWG file selection.
ViStatus _VI_FUNC rssfd_QueryDownstreamARBConfiguration(
	ViSession	instrSession,
	ViBoolean*	transmit,
	ViReal64*	frequency,
	ViReal64*	power,
	ViInt32*	triggerMode,
	ViReal64*	outputDelay,
	ViInt32*	interBurstGap,
	ViChar	arbFile[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     ext_buf=VI_NULL;
    ViChar*     p2buf;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

   
    checkErr (viPrintf (instrSession, "READ:CHAN:ARB:CONF?\n"));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, NULL));

	//STAT ON FREQ 600.000000e6 POW 20.5 MODE CONT DEL 0 IBG 0 SEL /Factory/waveform/off.wv

    p2buf = strtok (ext_buf, " ");

    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *transmit =  rssfd_StringIndex (offOnArr, p2buf) == VI_TRUE;
  
  	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *frequency = atof (p2buf);
  
    p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *power = atof (p2buf);
   
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *triggerMode =  rssfd_StringIndex (triggerModeArr, p2buf);
   
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *outputDelay = atof (p2buf);
    
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
	if (p2buf != NULL)
		*interBurstGap = atoi (p2buf);
	 
	   
    p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " \n");
	if (p2buf != NULL)
  		strcpy (arbFile, p2buf); 
    
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    

	return error;
}



/// HIFN  Reads the current DOCSIS 3.1 channel configuration for a specified 
/// HIFN  channel.
/// HIFN     
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     READ:CHANnel<Channel>:DOCsis:CONFig?
/// HIFN     
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Configuration'.
/// HIPAR configuration/Returns the DOCSIS 3.1 Channel Configuration.
ViStatus _VI_FUNC rssfd_QueryDOCSIS31ChannelConfiguration(
	ViSession	instrSession,
	ViInt32		bufferSize,
	ViChar		configuration[]
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar   	*p2data  = NULL;
    ViUInt32    count = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckErr (viPrintf(instrSession, "READ:CHAN:DOC:CONF?\n"));
	
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
	
    memcpy (configuration, p2data, ((ViUInt32) bufferSize<count) ? (ViUInt32) bufferSize : count);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns data rate in Mbps.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOCSIS_31_PROFILE_CHANNEL_DATA_RATE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     READ:CHANnel:PROFile<Profile>:DRATe?
/// HIFN     
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR dataRate/Returns the DOCSIS 3.1 profile date rate.
ViStatus _VI_FUNC rssfd_QueryDOCSIS31ProfileDataRate(
	ViSession	instrSession,
	ViInt32		profile,	
	ViReal64*	dataRate
)
{
	
	ViStatus    error = VI_SUCCESS;
	ViChar    rep_cap[15] = "";

    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

   
    if (rssfd_invalidViInt32Range (profile, 1, 4) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Profile");
    }	

   
    viCheckParm (rssfd_GetAttributeRepCapName (instrSession,
                                                 RSSFD_ATTR_DOCSIS_31_PROFILE_CHANNEL_DATA_RATE,
                                                 "Profile",
                                                 profile - 1,
                                                 10,
                                                 rep_cap), 2, "Profile");    

    viCheckParm (rssfd_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSSFD_ATTR_DOCSIS_31_PROFILE_CHANNEL_DATA_RATE,
                                                dataRate), 3, "Data Rate"); 
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the DOCSIS 3.0 channel settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOCSIS30_TRANSMIT
/// HIFN     RSSFD_ATTR_DOCSIS30_SOURCE
/// HIFN     RSSFD_ATTR_DOCSIS30_FREQUENCY
/// HIFN     RSSFD_ATTR_DOCSIS30_POWER
/// HIFN     RSSFD_ATTR_SYSTEM_CHANNEL_PORT
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MODulator:CHANnel:STATe
/// HIFN     MODulator:CHANnel:SOURce
/// HIFN     POWer:CHANnel:LEVel
/// HIFN     FREQuency:CHANnel:FREQuency
/// HIFN     SYSTem:CHANnel:PORT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transmit/Switches the modulator on or off for a specified channel.
/// HIPAR source/Selects the modulation source for a specified channel.
/// HIPAR power/Sets the power level for a specified channel.
/// HIPAR frequency/Sets the frequency for a specified channel.
/// HIPAR port/Sets the UDP port for a specified channel. Each DOCSIS 3.0 and DOCSIS 
/// HIPAR port/3.1 channel must be assigned a unique port number.
ViStatus _VI_FUNC rssfd_DownstreamDOCSIS30ChannelConfiguration (ViSession instrSession,
                                                         ViBoolean transmit,
                                                         ViInt32 source,
                                                         ViReal64 power,
                                                         ViReal64 frequency,
                                                         ViInt32 port)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	
     viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_DOCSIS30_TRANSMIT,
                                                 transmit), 2, "Transmit");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS30_SOURCE,
                                               source), 3, "Source");
	
    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_DOCSIS30_FREQUENCY,
                                                frequency), 4, "Frequency");
		
    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_DOCSIS30_POWER,
                                                power), 5, "Power");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_SYSTEM_CHANNEL_PORT,
                                               port), 6, "Port");

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the DOCSIS 3.0 block settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOCSIS30_BLOCK_TYPE
/// HIFN     RSSFD_ATTR_DOCSIS30_BLOCK_SYMBOL_RATE
/// HIFN     RSSFD_ATTR_DOCSIS30_BLOCK_CONSTELLATION
/// HIFN     RSSFD_ATTR_DOCSIS30_BLOCK_INTERLEAVER_MODE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     BLOCk:TYPE
/// HIFN     BLOCk:SYMBol
/// HIFN     BLOCk:CONStellation
/// HIFN     BLOCk:INTerleaver
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR type/Sets the type of modulation to use for the associated channel
/// HIPAR symbolRate/Sets the symbol rate for a specified baseband block. Note that 
/// HIPAR symbolRate/Reed-Solomon encoding is always on.
/// HIPAR constellation/Sets the constellation or modulation type for a specified baseband 
/// HIPAR constellation/block.
/// HIPAR interleaverMode/Sets the interleaver mode for a specified baseband block. Each 
/// HIPAR interleaverMode/interleaver mode is specified by a depth and delay (I,J) pair, where 
/// HIPAR interleaverMode/'I' is the interleaver depth or number of taps, and 'J' is the base 
/// HIPAR interleaverMode/delay or increment
ViStatus _VI_FUNC rssfd_DownstreamDOCSIS30BlockConfiguration (ViSession instrSession,
                                                         ViInt32 type,
                                                         ViReal64 symbolRate,
                                                         ViInt32 constellation,
                                                         ViInt32 interleaverMode
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS30_BLOCK_TYPE,
                                               type), 2, "Type");
	
    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_DOCSIS30_BLOCK_SYMBOL_RATE,
                                                symbolRate), 3, "Symbol Rate");
	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS30_BLOCK_CONSTELLATION,
                                               constellation), 4, "Constellation");
	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOCSIS30_BLOCK_INTERLEAVER_MODE,
                                               interleaverMode), 5, "Interleaver Mode");	


    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}



/// HIFN  If the external IP data source was selected, this query responds with 
/// HIFN  the amount of bits per second that are being consumed by the channel. 
/// HIFN  In the case of a DOCSIS 3.1 channel, the data rate is on a per profile 
/// HIFN  basis, as each profile can be supplied with data from an external 
/// HIFN  source. This query is useful for determining if the network settings 
/// HIFN  are working correctly and the channel is receiving the streamed network 
/// HIFN  data.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOCSIS30_DATA_RATE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     READ:CHANnel:DRATe?
/// HIFN     
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR dataRate/Returns the DOCSIS 3.0 date rate.
ViStatus _VI_FUNC rssfd_QueryDataRate(
	ViSession	instrSession,
	ViReal64*	dataRate
)
{
	
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

   
    viCheckParm (rssfd_GetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_DOCSIS30_DATA_RATE,
                                                dataRate), 2, "Data Rate"); 
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Reads the current channel and baseband block configuration for a 
/// HIFN  specified channel.
/// HIFN     
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     READ:CHANnel:CONFig?
/// HIFN     
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Returns state.
/// HIPAR source/Returns the modulation source for a specified channel.
/// HIPAR frequency/Returns the frequency for a specified channel.
/// HIPAR power/Returns the power level for a specified channel.
/// HIPAR port/Returns the UDP port for a specified channel.
ViStatus _VI_FUNC rssfd_QueryChannelConfiguration(
	ViSession	instrSession,
	ViBoolean*	state,
	ViInt32*	source,
	ViReal64*	frequency,
	ViReal64*	power,
	ViInt32*	port
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     ext_buf=VI_NULL;
    ViChar*     p2buf;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

   
    checkErr (viPrintf (instrSession, "READ:CHAN:CONF?\n"));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, NULL));

	//STAT ON SOUR PN23 FREQ 60.000000e6 POW 20.5 PORT 49152

    p2buf = strtok (ext_buf, " ");

    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *state =  rssfd_StringIndex (offOnArr, p2buf) == VI_TRUE;
    p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *source =  rssfd_StringIndex (sourceChannelArr, p2buf);
    p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *frequency = atof (p2buf);
    p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *power = atof (p2buf);
    p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " \n");
    if (p2buf != NULL)
        *port = atoi (p2buf);
    
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}





/// HIFN  Reads the current configuration for a specified baseband block.
/// HIFN     
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     READ:BLOCk:CONFig?
/// HIFN     
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR block/This control selects the block.
/// HIPAR symnolRate/Returns the symbol rate for a specified baseband block.
/// HIPAR constellation/Returns the constellation or modulation type for a specified baseband 
/// HIPAR constellation/block.
/// HIPAR interleaverDepth/Returns the interleaver depth (I).
/// HIPAR interleaverDelay/Returns the base delay (J).

ViStatus _VI_FUNC rssfd_QueryBlockConfiguration(
	ViSession	instrSession,
	ViInt32*	modulationType, 
	ViReal64*	symbolRate,
	ViInt32*	constellation,
	ViInt32*	interleaver,
	ViReal64*	rollOff
	
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     ext_buf=VI_NULL;
    ViChar*     p2buf;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (viPrintf (instrSession, "READ:BLOC:CONF?\n" ));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, NULL));
    
//MOD J83B SYMB 5.056946e6 CONS QAM64 INT (8,16) ROLL 0.12

	p2buf = strtok (ext_buf, " ");
    	
	p2buf = strtok (NULL, " ");
	if (p2buf != NULL)
        *modulationType = rssfd_StringIndex (modulationTypeArr, p2buf);
		
	p2buf = strtok (NULL, " ");
	p2buf = strtok (NULL, " ");  
	if (p2buf != NULL)
        *symbolRate = atof (p2buf);
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *constellation =  rssfd_StringIndex (constellationArr, p2buf);
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *interleaver =  rssfd_StringIndex (interleaverArr, p2buf);
    
	p2buf = strtok (NULL, " "); 
	p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
	    *rollOff = atof (p2buf); 
      

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Returns useful (maximum) data rate in Mbps, on a per block basis.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOCSIS30_USEFUL_DATA_RATE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     READ:BLOCkUSEFul?
/// HIFN     
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR usefulDataRate/Useful (maximum) data rate.
ViStatus _VI_FUNC rssfd_QueryUsefulDataRate(
	ViSession	instrSession,
	ViReal64*	usefulDataRate
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

   
    viCheckParm (rssfd_GetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_DOCSIS30_USEFUL_DATA_RATE,
                                                usefulDataRate), 2, "Useful Data Rate"); 
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}



/// HIFN  This function configures the MPEG transport stream generator settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_TSGEN_TRANSPORT_SELECT
/// HIFN     RSSFD_ATTR_TSGEN_CONTINUITY
/// HIFN     RSSFD_ATTR_TSGEN_PCR
/// HIFN     RSSFD_ATTR_TSGEN_TDT
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     TSGenerator:TRANsport:SEL
/// HIFN     TSGenerator:CONTinuity
/// HIFN     TSGenerator:PCR
/// HIFN     TSGenerator:TDT
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileToPlay/Selects and loads am MPEG file for an input source.
/// HIPAR continuity/Configure the continuity counter correction for the transport stream 
/// HIPAR continuity/playback.
/// HIPAR pcr/Configure the PCR, DTS/PTS correction for the transport stream 
/// HIPAR pcr/playback.
/// HIPAR tdt/Configure the TDT/TOT correction for the transport stream playback.
ViStatus _VI_FUNC rssfd_ConfigureMPEGTSGenerator(
	ViSession	instrSession,
	ViBoolean	continuity,
	ViBoolean	pcr,
	ViBoolean	tdt,
	ViString	fileToPlay
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));


    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSSFD_ATTR_TSGEN_CONTINUITY,
                                                 continuity), 2, "Continuity");

    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSSFD_ATTR_TSGEN_PCR,
                                                 pcr), 3, "PCR");
	
    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSSFD_ATTR_TSGEN_TDT,
                                                 tdt), 4, "TDT");
	viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                RSSFD_ATTR_TSGEN_TRANSPORT_SELECT,
                                                fileToPlay), 5, "File to Play");
	
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the upstream settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_UPSTREAM_TRANSMIT
/// HIFN     RSSFD_ATTR_UPSTREAM_TIMING_SYNCHRONIZATION
/// HIFN     RSSFD_ATTR_UPSTREAM_MODULATION_TYPE
/// HIFN     RSSFD_ATTR_UPSTREAM_POWER
/// HIFN     RSSFD_ATTR_UPSTREAM_FREQUENCY
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MODulator:CHANnel:UPstream:STATe
/// HIFN     MODulator:CHANnel:UPstream:TSYNchronization
/// HIFN     MODulator:CHANnel:UPstream:MODulation
/// HIFN     POWer:CHANnel:UPstream:LEVel
/// HIFN     FREQuency:CHANnel:UPstream:FREQuency
/// HIFN     
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transmit/Switches the modulator on or off for a specified channel.
/// HIPAR timingSync/Specify when the minislot/frame and DOCSIS timestamp parameters get 
/// HIPAR timingSync/written to the internal clocking system. Automatic synchronizes the 
/// HIPAR timingSync/time as soon as the channel is turned on. Setting this parameter to a 
/// HIPAR timingSync/value of triggered causes the unit to wait for a trigger to seed and 
/// HIPAR timingSync/start the internal clocking system.
/// HIPAR modulationType/Specify the modulation type for the upstream DOCSIS 3.0 channel. 
/// HIPAR modulationType/Further modulation details are configured used the modulation specific 
/// HIPAR modulationType/SCPI commands listed under the ATDMA and SCDMA sections.
/// HIPAR frequency/Sets the frequency for a specified channel.
/// HIPAR power/Sets the power level for a specified channel.
ViStatus _VI_FUNC rssfd_ConfigureUpstream(
 										   ViSession instrSession,
                                           ViBoolean transmit, ViInt32 timingSync,
                                           ViInt32 modulationType,
                                           ViReal64 frequency, ViReal64 power)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
			
    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_UPSTREAM_TRANSMIT,
                                                 transmit), 2, "Transmit");

	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_UPSTREAM_TIMING_SYNCHRONIZATION,
                                                 timingSync), 3, "Timing Synchronization");

	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_UPSTREAM_MODULATION_TYPE,
                                                 modulationType),4, "Modulation Type");

    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_UPSTREAM_FREQUENCY,
                                                frequency), 5, "Frequency");

	viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_UPSTREAM_POWER,
                                                power), 6, "Power");
    
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Configures source.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_UPSTREAM_SOURCE
/// HIFN     RSSFD_ATTR_UPSTREAM_GRANT_SIZE
/// HIFN     RSSFD_ATTR_UPSTREAM_CHANNEL_PORT
/// HIFN     RSSFD_ATTR_UPSTREAM_PLAYOUT
/// HIFN     RSSFD_ATTR_UPSTREAM_PERIODIC_START_FRAME
/// HIFN     RSSFD_ATTR_UPSTREAM_PERIODIC_FRAME_REPEAT_INTERVAL
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MODulator:CHANnel:UPstream:SOURce
/// HIFN     MODulator:CHANnel:UPstream:GRANt
/// HIFN     SYSTem:CHANnel:UPstream:PORT
/// HIFN     MODulator:CHANnel:UPstream:PLAYout
/// HIFN     MODulator:CHANnel:UPstream:STARt
/// HIFN     MODulator:CHANnel:UPstream:RINTerval
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/Sets the input data source to be modulated onto the specified channel.
/// HIPAR grantSize/The grant size is required when playing out pseudo random data to 
/// HIPAR grantSize/determine how many minislots the data should span.
/// HIPAR port/Sets the UDP port for a specified channel. Port number must be unique 
/// HIPAR port/between transmitting channels that use the ports.
/// HIPAR playout/Specify how the playout of the channel is configured. When a source of 
/// HIPAR playout/SFP is used, the available playout selections are IMMediate and 
/// HIPAR playout/DYNamic, the remaining sources offer playout types of PERiodic or 
/// HIPAR playout/TRIGgered.
/// HIPAR start/This parameter is used when a playout option of PERiodic is used. It 
/// HIPAR start/determines the frame that the data is played into. When dealing with a 
/// HIPAR start/DOCSIS 3.0 ATDMa channel, this parameter is the minislot to inject the 
/// HIPAR start/data into. This parameter is not used if a
/// HIPAR repeat/This parameter is used when a playout option of PERiodic is used. It 
/// HIPAR repeat/determines the periodicity of the data playout. This parameter 
/// HIPAR repeat/accompanies the STARt parameter toallow bursts to repeat after a set 
/// HIPAR repeat/number of frames. To playout a single burst  this parameter should be 
/// HIPAR repeat/set to 0.
ViStatus _VI_FUNC rssfd_ConfigureUpstreamSourceOptions(
	ViSession	instrSession,
	ViInt32	source,
	ViInt32	grantSize,
	ViInt32	port,
	ViInt32	playout,
	ViInt32	start,
	ViInt32	repeat
)
{
    ViStatus  error = VI_SUCCESS;
   	
	    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_UPSTREAM_SOURCE,
                                               source), 2, "Source");
	
	if (source != RSSFD_VAL_UPSTREAM_SOURCE_CW) {
    
		if (source ==  RSSFD_VAL_UPSTREAM_SOURCE_PRBS) {
		viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_UPSTREAM_GRANT_SIZE,
                                               grantSize), 3, "Grant Size");
		}
    
	
		if (source ==  RSSFD_VAL_UPSTREAM_SOURCE_SFP) {
				viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
	                                               RSSFD_ATTR_UPSTREAM_CHANNEL_PORT,
	                                               port), 4, "Port");
		}
		  
		viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_UPSTREAM_PLAYOUT,
                                               playout), 5, "Playout");
		

		 if (playout == RSSFD_VAL_UPSTREAM_PLAYOUT_PERIODIC) {
			 	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
	                                               RSSFD_ATTR_UPSTREAM_PERIODIC_START_FRAME,
	                                               start), 6, "Start");
		 		viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
	                                               RSSFD_ATTR_UPSTREAM_PERIODIC_FRAME_REPEAT_INTERVAL,
	                                               repeat), 7, "Repeat");
		 }
	
		
	 }

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Returns data rate in Mbps.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_UPSTREAM_CHANNEL_DATA_RATE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     READ:CHANnel:UPstream:DRATe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR dataRate/Returns data rate in Mbps.
ViStatus _VI_FUNC rssfd_QueryUpstreamDataRate(
	ViSession	instrSession,
	ViReal64*	dataRate
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_GetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_UPSTREAM_CHANNEL_DATA_RATE,
                                                dataRate), 2, "Data Rate"); 
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Reads the current upstream configuration for a specified channel.
/// HIFN     
/// HIFN     This command shows the current channels parameter configuration for 
/// HIFN  upstream DOCSIS 3.0 transmission. The query responds with a list of 
/// HIFN  channel parameters in the following order. Additional modulation 
/// HIFN  details must be queried separately by the appropriate type of 
/// HIFN  modulation selected on the channel.
/// HIFN      STAT Output state
/// HIFN      TSYN Time synchronization mode
/// HIFN      TYPE Burst modulation type
/// HIFN      SOUR Input data source
/// HIFN      PORT SFP+ UDP port number
/// HIFN      GRAN Grant size in minislots
/// HIFN      PLAY Playout configuration
/// HIFN      STAR Start frame for period playout
/// HIFN      RINT Repeat interval for period playout
/// HIFN      FREQ Frequency
/// HIFN      POW Power
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     READ:CHANnel:UPstream:CONFig?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Returns state.
/// HIPAR timingSync/Returns the time synchronization mode.
/// HIPAR modulationType/Returns the burst modulation type.
/// HIPAR source/Returns the input data source.
/// HIPAR port/Returns the SFP+ UDP port number.
/// HIPAR grantSize/Returns the grant size in minislots
/// HIPAR playout/Returns the playout configuration
/// HIPAR start/Returns the start frame for period playout
/// HIPAR repeat/Returns the repeat interval for period playout
/// HIPAR frequency/Returns the frequency for a specified channel.
/// HIPAR power/Returns the power level for a specified channel.
ViStatus _VI_FUNC rssfd_QueryUpstreamChannelConfiguration(
	ViSession	instrSession,
	ViBoolean*	transmit,
	ViInt32*	timingSync,
	ViInt32*	modulationType,
	ViInt32*	source,
	ViInt32*	port,
	ViInt32*	grantSize,
	ViInt32*	playout,
	ViInt32*	start,
	ViInt32*	repeat,
	ViReal64*	frequency,
	ViReal64*	power
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar*     ext_buf=VI_NULL;
    ViChar*     p2buf;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

   
	checkErr (viPrintf (instrSession, "READ:CHAN:UPSTREAM:CONF?\n"));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, NULL));
   
	//STAT ON TSYN AUTO TYPE ATDM SOUR SFP PORT 1000 GRAN 0 PLAY IMM STAR 0 RINT 0 FREQ 5.000000e6 POW 20.5 
    
    p2buf = strtok (ext_buf, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *transmit =  rssfd_StringIndex (offOnArr, p2buf) == VI_TRUE;
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *timingSync =  rssfd_StringIndex (timingSyncModeArr, p2buf);
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");;
    if (p2buf != NULL)
        *modulationType =  rssfd_StringIndex (modulationArr, p2buf);
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *source =  rssfd_StringIndex (sourceArr, p2buf);
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *port = atoi (p2buf);
		
	p2buf = strtok (NULL, " ");
	p2buf = strtok (NULL, " ");;
    if (p2buf != NULL)
        *grantSize = atoi (p2buf); 
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *playout = rssfd_StringIndex (playoutArr, p2buf);
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *start = atoi (p2buf); 
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *repeat = atoi (p2buf); 
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *frequency = atof (p2buf);
	
   	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " \n");
    if (p2buf != NULL)
        *power = atof (p2buf);

   
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}



/// HIFN  This function configures the minislot and bFurst size.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_ATDMA_MINISLOT_SIZE
/// HIFN     RSSFD_ATTR_ATDMA_SYMBOL_RATE
/// HIFN     RSSFD_ATTR_ATDMA_NUMBER_OF_MINISLOTS
/// HIFN     RSSFD_ATTR_ATDMA_GUARD_TIME_SIZE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     ATDMa:MINIslot:SIZE
/// HIFN     ATDMa:MINIslot:SYMBol
/// HIFN     ATDMa:MINIslot:NUMBer
/// HIFN     ATDMa:GuaRD:SIZE
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR minislotSize/Configures the size of a single minislot.
/// HIPAR symbolRate/Configures the symbol rate as multiples of the base rate.
/// HIPAR numberOfMinislots/Configures the number of minislots in the generated waveform.
/// HIPAR guardTimeSize/The guard time based on units of modulation intervals.
ViStatus _VI_FUNC rssfd_ConfigureATDMAMinislot (
	ViSession instrSession,
	ViInt32 minislotSize,
	ViInt32 symbolRate,
	ViInt32 guardTimeSize,
	ViBoolean fillRemainingMinislots
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_ATDMA_MINISLOT_SIZE,
                                                 minislotSize), 2, "Minislot Size");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_ATDMA_SYMBOL_RATE,
                                                symbolRate), 3, "Symbol Rate");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_ATDMA_GUARD_TIME_SIZE,
                                                guardTimeSize), 4, "Guard Time Size");
	
	viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_ATDMA_FILL_REMAINING_MINISLOTS,
                                                 fillRemainingMinislots), 5, "Fill Remaining Minislots");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}



/// HIFN  This function configures the constellation type for the modulation scheme.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_ATDMA_CONSTELLATION
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     ATDMa:MODulation
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR modulationType/The constellation type for the modulation scheme.
ViStatus _VI_FUNC rssfd_ConfigureATDMAModulation (
	ViSession instrSession,
	ViInt32 constellation
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_ATDMA_CONSTELLATION,
                                                 constellation), 2, "Constellation");

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the preamble settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_ATDMA_PREAMBLE_PATTERN
/// HIFN     RSSFD_ATTR_ATDMA_PREAMBLE_LENGTH
/// HIFN     RSSFD_ATTR_ATDMA_PREAMBLE_VALUE_OFFSET
/// HIFN     RSSFD_ATTR_ATDMA_PREAMBLE_TYPE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     ATDMa:PREAmble:PATTern
/// HIFN     ATDMa:PREAmble:LENGth
/// HIFN     ATDMa:PREAmble:OFFSet
/// HIFN     ATDMa:PREAmble:TYPE
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR preamblePattern/The preamble pattern for the data
/// HIPAR preambleLength/Configures the preamble length.
/// HIPAR preambleValueOffset/Configures the starting offset of the preamble pattern.
/// HIPAR preambleType/The type of the preamble pattern
ViStatus _VI_FUNC rssfd_ConfigureATDMAPreamble (
	ViSession instrSession,
	ViString preamblePattern,
	ViInt32 preambleLength,
	ViInt32 preambleValueOffset,
	ViInt32 preambleType
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                 RSSFD_ATTR_ATDMA_PREAMBLE_PATTERN,
                                                 preamblePattern), 2, "Preamble Pattern");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_ATDMA_PREAMBLE_LENGTH,
                                                preambleLength), 3, "Preamble Length");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_ATDMA_PREAMBLE_VALUE_OFFSET,
                                                preambleValueOffset), 4, "Preamble Value Offset");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_ATDMA_PREAMBLE_TYPE,
                                                preambleType), 5, "Preamble Type");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the FEC settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_ATDMA_FORWARD_ERROR_CORRECTION
/// HIFN     RSSFD_ATTR_ATDMA_FECK
/// HIFN     RSSFD_ATTR_ATDMA_LAST_CODEWORD_LENGTH
/// HIFN     RSSFD_ATTR_ATDMA_RS_INTERLEAVER_MODE
/// HIFN     RSSFD_ATTR_ATDMA_RS_INTERLEAVER_DEPTH
/// HIFN     RSSFD_ATTR_ATDMA_RS_INTERLEAVER_BLOCK_SIZE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     ATDMa:FECT
/// HIFN     ATDMa:FECK
/// HIFN     ATDMa:FEC:LENGth
/// HIFN     ATDMa:INTerleaver:MODE
/// HIFN     ATDMa:INTerleaver:DEPTh
/// HIFN     ATDMa:INTerleaver:SIZE
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR FECErrorCorrection/Specify error correction capability of the code.
/// HIPAR FECErrCorrInformationBytes/Specify the number of information bytes in the FEC code word.
/// HIPAR lastCodewordLength/Determines if the last code word is shortened to fit, or left as is.
/// HIPAR rSInterleaverMode/Modifies how the interleaver is used.
/// HIPAR rSInterleaverDepth/Configure the R-S interleaver depth.
/// HIPAR rSInterleaverBlockSize/Configure the R-S interleaver block size.
ViStatus _VI_FUNC rssfd_ConfigureATDMAFEC (
	ViSession instrSession,
	ViInt32 FECErrorCorrection,
	ViInt32 FECErrCorrInformationBytes,
	ViInt32 lastCodewordLength,
	ViInt32 rSInterleaverMode,
	ViInt32 rSInterleaverDepth,
	ViInt32 rSInterleaverBlockSize
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_ATDMA_FORWARD_ERROR_CORRECTION,
                                                 FECErrorCorrection), 2, "FEC Error Correction");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_ATDMA_FECK,
                                                FECErrCorrInformationBytes), 3, "FEC Err Corr Information Bytes");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_ATDMA_LAST_CODEWORD_LENGTH,
                                                lastCodewordLength), 4, "Last Codeword Length");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_ATDMA_RS_INTERLEAVER_MODE,
                                                rSInterleaverMode), 5, "RS Interleaver Mode");
	
	if(rSInterleaverMode == RSSFD_VAL_ATDMA_INTERLEAVER_MODE_FIXED)
	{
	    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_ATDMA_RS_INTERLEAVER_DEPTH,
                                                rSInterleaverDepth), 6, "RS Interleaver Depth");
	}

	if(rSInterleaverMode == RSSFD_VAL_ATDMA_INTERLEAVER_MODE_DYNAMIC)
	{
	    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_ATDMA_RS_INTERLEAVER_BLOCK_SIZE,
                                                rSInterleaverBlockSize), 7, "RS Interleaver Block Size");
	}
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the scrambler settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_ATDMA_SCRAMBLER_STATE
/// HIFN     RSSFD_ATTR_ATDMA_SCRAMBLER_SEED
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     ATDMa:SCRambler:STATe
/// HIFN     ATDMa:SCRambler:SEED
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR scramblerState/Enable or disable the data randomizer.
/// HIPAR scramblerSeed/Configure the seed to use for the data randomizer.
ViStatus _VI_FUNC rssfd_ConfigureATDMAScrambler (
	ViSession instrSession,
	ViBoolean scramblerState,
	ViString scramblerSeed
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_ATDMA_SCRAMBLER_STATE,
                                                 scramblerState), 2, "Scrambler State");

    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                RSSFD_ATTR_ATDMA_SCRAMBLER_SEED,
                                                scramblerSeed), 3, "Scrambler Seed");

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the minislot and burst size.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_SCDMA_SYMBOL_RATE
/// HIFN     RSSFD_ATTR_SCDMA_CODES
/// HIFN     RSSFD_ATTR_SCDMA_SPREADING
/// HIFN     RSSFD_ATTR_SCDMA_ACTIVE_CODES
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SCDMa:MINIslot:SYMBol
/// HIFN     SCDMa:MINIslot:CODes
/// HIFN     SCDMa:MINIslot:SPReading
/// HIFN     SCDMa:MINIslot:ACODes
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR symbolRate/Configures the symbol rate as multiples of the base rate.
/// HIPAR codes/Configure the number of codes in each minislot.
/// HIPAR spreading/Configure the number of spreading intervals in each frame.
/// HIPAR activeCodes/Configure the number of active codes.
ViStatus _VI_FUNC rssfd_ConfigureSCDMAMinislot (
	ViSession instrSession,
	ViInt32 symbolRate,
	ViInt32 activeCodes,
	ViInt32 spreading,
	ViInt32 codes,
	ViInt32 minislotLoadingDutyCycle
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_SCDMA_SYMBOL_RATE,
                                                 symbolRate), 2, "Symbol Rate");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_SCDMA_ACTIVE_CODES,
                                                activeCodes), 3, "Active Codes");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_SCDMA_SPREADING,
                                                spreading), 4, "Spreading");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_SCDMA_CODES,
                                                codes), 5, "Codes");

	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_SCDMA_MINISLOT_LOADING_DUTY_CYCLE,
                                                minislotLoadingDutyCycle), 6, "Minislot Loading Duty Cycle");	
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the constellation type for the modulation scheme.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_SCDMA_CONSTELLATION
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SCDMa:CONStellation
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR modulationType/The constellation type for the modulation scheme.
ViStatus _VI_FUNC rssfd_ConfigureSCDMAModulation (
	ViSession instrSession,
	ViInt32 constellation
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_SCDMA_CONSTELLATION,
                                                 constellation), 2, "Constellation");

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the preamble settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_SCDMA_PREAMBLE_PATTERN
/// HIFN     RSSFD_ATTR_SCDMA_PREAMBLE_LENGTH
/// HIFN     RSSFD_ATTR_SCDMA_PREAMBLE_VALUE_OFFSET
/// HIFN     RSSFD_ATTR_SCDMA_PREAMBLE_TYPE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SCDMa:PREAmble:PATTern
/// HIFN     SCDMa:PREAmble:LENGth
/// HIFN     SCDMa:PREAmble:OFFSet
/// HIFN     SCDMa:PREAmble:TYPE
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR preamblePattern/The preamble pattern for the data,
/// HIPAR preambleLength/Configures the preamble length.
/// HIPAR preambleValueOffset/Configures the starting offset of the preamble pattern.
/// HIPAR preambleType/The type of the preamble pattern,
ViStatus _VI_FUNC rssfd_ConfigureSCDMAPreamble (
	ViSession instrSession,
	ViString preamblePattern,
	ViInt32 preambleLength,
	ViInt32 preambleValueOffset,
	ViInt32 preambleType
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                 RSSFD_ATTR_SCDMA_PREAMBLE_PATTERN,
                                                 preamblePattern), 2, "Preamble Pattern");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_SCDMA_PREAMBLE_LENGTH,
                                                preambleLength), 3, "Preamble Length");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_SCDMA_PREAMBLE_VALUE_OFFSET,
                                                preambleValueOffset), 4, "Preamble Value Offset");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_SCDMA_PREAMBLE_TYPE,
                                                preambleType), 5, "Preamble Type");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the FEC settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_SCDMA_FORWARD_ERROR_CORRECTION
/// HIFN     RSSFD_ATTR_SCDMA_FECK
/// HIFN     RSSFD_ATTR_SCDMA_LAST_CODEWORD_LENGTH
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SCDMa:FECT
/// HIFN     SCDMa:FECK
/// HIFN     SCDMa:FEC:LENGth
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR FECErrorCorrection/Specify error correction capability of the code.
/// HIPAR FECErrCorrInformationBytes/Specify the number of information bytes in the FEC code word.
/// HIPAR lastCodewordLength/Determines if the last code word is shortened to fit, or left as is.
ViStatus _VI_FUNC rssfd_ConfigureSCDMAFEC (
	ViSession instrSession,
	ViInt32 FECErrorCorrection,
	ViInt32 FECErrCorrInformationBytes,
	ViInt32 lastCodewordLength
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_SCDMA_FORWARD_ERROR_CORRECTION,
                                                 FECErrorCorrection), 2, "FEC Error Correction");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_SCDMA_FECK,
                                                FECErrCorrInformationBytes), 3, "FEC Err Corr Information Bytes");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_SCDMA_LAST_CODEWORD_LENGTH,
                                                lastCodewordLength), 4, "Last Codeword Length");

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the Framer and Spreader..
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_SCDMA_SYMBOL_INTERLEAVER_STEP_SIZE
/// HIFN     RSSFD_ATTR_SCDMA_CODE_HOPPING
/// HIFN     RSSFD_ATTR_SCDMA_SPREADER_STATE
/// HIFN     RSSFD_ATTR_SCDMA_CODES_PER_SUBFRAME
/// HIFN     RSSFD_ATTR_SCDMA_CODE_HOPPING_SEED
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SCDMa:SISSize
/// HIFN     SCDMa:CHOPping:STATe
/// HIFN     SCDMa:SPReading:STATe
/// HIFN     SCDMa:CPSubframe
/// HIFN     SCDMa:CHOPping:SEED
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR symbolInterleaverStepSize/Configures the placement of the preamble and coded TCM
/// HIPAR symbolInterleaverStepSize/subsymbols with the transmission subframes.
/// HIPAR codeHopping/Enable or disable the code hopping feature.
/// HIPAR spreaderState/Enable or disable the spreader.
/// HIPAR codesPerSubframe/Configure the number of codes per subframe.
/// HIPAR codeHoppingSeed/Configure the code hopping seed. Leaving blank will result in
/// HIPAR codeHoppingSeed/using the default seed.
ViStatus _VI_FUNC rssfd_ConfigureSCDMAFramerSpreader (
	ViSession instrSession,
	ViInt32 symbolInterleaverStepSize,
	ViBoolean codeHopping,
	ViBoolean spreaderState,
	ViInt32 codesPerSubframe,
	ViString codeHoppingSeed
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_SCDMA_SYMBOL_INTERLEAVER_STEP_SIZE,
                                                 symbolInterleaverStepSize), 2, "Symbol Interleaver Step Size");
	
    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_SCDMA_CODE_HOPPING,
                                                 codeHopping), 3, "Code Hopping");

    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_SCDMA_SPREADER_STATE,
                                                 spreaderState), 4, "Spreader State");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_SCDMA_CODES_PER_SUBFRAME,
                                                codesPerSubframe), 5, "Codes Per Subframe");

    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                RSSFD_ATTR_SCDMA_CODE_HOPPING_SEED,
                                                codeHoppingSeed), 6, "Code Hopping Seed");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the scrambler settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_SCDMA_SCRAMBLER_STATE
/// HIFN     RSSFD_ATTR_SCDMA_SCRAMBLER_SEED
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SCDMa:SCRambler:STATe
/// HIFN     SCDMa:SCRambler:SEED
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR scramblerState/Enable or disable the data randomizer.
/// HIPAR scramblerSeed/Configure the seed to use for the data randomizer.
ViStatus _VI_FUNC rssfd_ConfigureSCDMAScrambler (
	ViSession instrSession,
	ViBoolean scramblerState,
	ViString scramblerSeed
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_SCDMA_SCRAMBLER_STATE,
                                                 scramblerState), 2, "Scrambler State");

    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                RSSFD_ATTR_SCDMA_SCRAMBLER_SEED,
                                                scramblerSeed), 3, "Scrambler Seed");

	checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the upstream DOCsis channel settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_UPSTREAM_DOCSIS_TRANSMIT
/// HIFN     RSSFD_ATTR_UPSTREAM_DOCSIS_TIMING_SYNCHRONIZATION
/// HIFN     RSSFD_ATTR_UPSTREAM_DOCSIS_BURST_TYPE
/// HIFN     RSSFD_ATTR_UPSTREAM_DOCSIS_FREQUENCY
/// HIFN     RSSFD_ATTR_UPSTREAM_DOCSIS_POWER
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MODulator:CHANnel:UPstream:DOCsis:STATe
/// HIFN     MODulator:CHANnel:UPstream:DOCsis:TSYNchronization
/// HIFN     MODulator:CHANnel:UPstream:DOCsis:BURSt
/// HIFN     FREQuency:CHANnel:UPstream:DOCsis:FREQuency
/// HIFN     POWer:CHANnel:UPstream:DOCsis:LEVe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transmit/Switches the modulator on or off for a specified channel.
/// HIPAR syncMode/Specify when the minislot/frame and DOCSIS timestamp parameters get 
/// HIPAR syncMode/written to the internal clocking system. Automatic synchronizes the 
/// HIPAR syncMode/time as soon as the channel is turned on. Setting this parameter to a 
/// HIPAR syncMode/value of triggered causes the unit to wait for a trigger to seed and 
/// HIPAR syncMode/start the internal clocking system.
/// HIPAR burstType/Specify the modulation burst type for the upstream DOCSIS 3.1 channel. 
/// HIPAR burstType/Further modulation details are configured used the modulation specific 
/// HIPAR burstType/SCPI commands listed under the OFDMA section.
/// HIPAR power/Sets the power level for a specified channel.
/// HIPAR frequency/Sets the frequency for a specified channel.
ViStatus _VI_FUNC rssfd_ConfigureUpstreamDOCsis(
	ViSession	instrSession,
	ViBoolean	transmit,
	ViInt32		syncMode,
	ViInt32		burstType,
	ViReal64	frequency,
	ViReal64	power
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
			
    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_UPSTREAM_DOCSIS_TRANSMIT,
                                                 transmit), 2, "Transmit");

	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_UPSTREAM_DOCSIS_TIMING_SYNCHRONIZATION,
                                                 syncMode), 3, "Sync Mode");

	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_UPSTREAM_DOCSIS_BURST_TYPE,
                                                 burstType),4, "Burst Type");

    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_UPSTREAM_DOCSIS_FREQUENCY,
                                                frequency), 5, "Frequency");

	viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_UPSTREAM_DOCSIS_POWER,
                                                power), 6, "Power");

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Configures source.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_UPSTREAM_DOCSIS_SOURCE
/// HIFN     RSSFD_ATTR_UPSTREAM_DOCSIS_GRANT_SIZE
/// HIFN     RSSFD_ATTR_UPSTREAM_DOCSIS_CHANNEL_PORT
/// HIFN     RSSFD_ATTR_UPSTREAM_DOCSIS_PLAYOUT
/// HIFN     RSSFD_ATTR_UPSTREAM_DOCSIS_PERIODIC_START_FRAME
/// HIFN     RSSFD_ATTR_UPSTREAM_DOCSIS_PERIODIC_FRAME_REPEAT_INTERVAL
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MODulator:CHANnel:UPstream:DOCsis:SOURce
/// HIFN     MODulator:CHANnel:UPstream:DOCsis:GRANt
/// HIFN     SYSTem:CHANnel:UPstream:DOCsis:PORT
/// HIFN     MODulator:CHANnel:UPstream:DOCsis:PLAYout
/// HIFN     MODulator:CHANnel:UPstream:DOCsis:STARt
/// HIFN     MODulator:CHANnel:UPstream:DOCsis:RINTerval
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/Sets the input data source to be modulated onto the specified channel.
/// HIPAR grantSize/The grant size is required when playing out pseudo random data to 
/// HIPAR grantSize/determine how many minislots the data should span.
/// HIPAR port/Sets the UDP port for a specified channel. Port number must be unique 
/// HIPAR port/between transmitting channels that use the ports.
/// HIPAR playout/Specify how the playout of the channel is configured. When a source of 
/// HIPAR playout/SFP is used, the available playout selections are IMMediate and 
/// HIPAR playout/DYNamic, the remaining sources offer playout types of PERiodic or 
/// HIPAR playout/TRIGgered.
/// HIPAR start/This parameter is used when a playout option of PERiodic is used. It 
/// HIPAR start/determines the frame that the data is played into. When dealing with a 
/// HIPAR start/DOCSIS 3.0 ATDMa channel, this parameter is the minislot to inject the 
/// HIPAR start/data into. This parameter is not used if a
/// HIPAR repeat/This parameter is used when a playout option of PERiodic is used. It 
/// HIPAR repeat/determines the periodicity of the data playout. This parameter 
/// HIPAR repeat/accompanies the STARt parameter toallow bursts to repeat after a set 
/// HIPAR repeat/number of frames. To playout a single burst  this parameter should be 
/// HIPAR repeat/set to 0.
ViStatus _VI_FUNC rssfd_ConfigureUpstreamDOCsisSourceOptions(
	ViSession	instrSession,
	ViInt32	source,
	ViInt32	grantSize,
	ViInt32	port,
	ViInt32	playout,
	ViInt32	start,
	ViInt32	repeat
)
{
    ViStatus  error = VI_SUCCESS;
    	
	    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_UPSTREAM_DOCSIS_SOURCE,
                                               source), 2, "Source");
	
	if (source != RSSFD_VAL_UPSTREAM_DOCSIS_SOURCE_CW) {
    
		if (source ==  RSSFD_VAL_UPSTREAM_SOURCE_PRBS) {
		viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_UPSTREAM_DOCSIS_GRANT_SIZE,
                                               grantSize), 3, "Grant Size");
		}
    
	
		if (source ==  RSSFD_VAL_UPSTREAM_DOCSIS_SOURCE_SFP) {
				viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
	                                               RSSFD_ATTR_UPSTREAM_DOCSIS_CHANNEL_PORT,
	                                               port), 4, "Port");
		}
		  
		viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_UPSTREAM_DOCSIS_PLAYOUT,
                                               playout), 5, "Playout");
		

		 if (playout == RSSFD_VAL_UPSTREAM_PLAYOUT_PERIODIC) {
			 	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
	                                               RSSFD_ATTR_UPSTREAM_DOCSIS_PERIODIC_START_FRAME,
	                                               start), 6, "Start");
		 		viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
	                                               RSSFD_ATTR_UPSTREAM_DOCSIS_PERIODIC_FRAME_REPEAT_INTERVAL,
	                                               repeat), 7, "Repeat");
		 }
	
		
	 }

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Returns data rate in Mbps.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_UPSTREAM_DOCSIS_CHANNEL_DATA_RATE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     READ:CHANnel:UPstream:DOCsis:DRATe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR dataRate/Returns data rate in Mbps.
ViStatus _VI_FUNC rssfd_QueryUpstreamDOCsisDataRate(
	ViSession	instrSession,
	ViReal64*	dataRate
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_GetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_UPSTREAM_DOCSIS_CHANNEL_DATA_RATE,
                                                dataRate), 2, "Data Rate"); 
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Reads the current upstream configuration for a specified channel.
/// HIFN     
/// HIFN     This command shows the current channels parameter configuration for 
/// HIFN  upstream DOCSIS 3.1 transmission. The query responds with a list of 
/// HIFN  channel parameter settings in the following order. Additional 
/// HIFN  parameters related to the burst type of the modulation scheme must be 
/// HIFN  queried separately.
/// HIFN      STAT Output state
/// HIFN      TSYN Time synchronization mode
/// HIFN      BURS Burst type generated
/// HIFN      SOUR Input data source
/// HIFN      PORT SFP UDP port number
/// HIFN      GRAN Grant size in minislots
/// HIFN      PLAY Playout configuration
/// HIFN      STAR Start frame for period playout
/// HIFN      RINT Repeat interval for period playout
/// HIFN      FREQ Frequency
/// HIFN      POW Power
/// HIFN     
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     READ:CHANnel:UPstream:DOCsis:CONFig?
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transmit/Returns state.
/// HIPAR power/Returns the power level for a specified channel.
/// HIPAR frequency/Returns the frequency for a specified channel.
/// HIPAR timingSync/Returns the time synchronization mode.
/// HIPAR burstType/Returns the burst modulation type.
/// HIPAR playout/Returns the playout configuration
/// HIPAR source/Returns the input data source.
/// HIPAR grantSize/Returns the grant size in minislots
/// HIPAR port/Returns the SFP+ UDP port number.
/// HIPAR start/Returns the start frame for period playout
/// HIPAR repeat/Returns the repeat interval for period playout
ViStatus _VI_FUNC rssfd_QueryUpstreamDOCsisChannelConfiguration(
	ViSession	instrSession,
	ViBoolean*	transmit,
	ViInt32*	timingSync,
	ViInt32*	burstType,
	ViInt32*	source,
	ViInt32*	port,
	ViInt32*	grantSize,
	ViInt32*	playout,
	ViInt32*	start,
	ViInt32*	repeat,
	ViReal64*	frequency,
	ViReal64*	power
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar*     ext_buf=VI_NULL;
    ViChar*     p2buf;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

   
	checkErr (viPrintf (instrSession, "READ:CHAN:UPSTREAM:DOCsis:CONF?\n"));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, NULL));
  
   //STAT ON TSYN AUTO BURS DMOD SOUR SFP PORT 1000 GRAN 0 PLAY IMM STAR 0 RINT 0 FREQ 5.000000e6 POW 20.5

      p2buf = strtok (ext_buf, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *transmit =  rssfd_StringIndex (offOnArr, p2buf) == VI_TRUE;
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *timingSync =  rssfd_StringIndex (timingSyncModeArr, p2buf);
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");;
    if (p2buf != NULL)
        *burstType =  rssfd_StringIndex (bursTypeArr, p2buf);
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *source =  rssfd_StringIndex (sourceArr, p2buf);
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *port = atoi (p2buf);
		
	p2buf = strtok (NULL, " ");
	p2buf = strtok (NULL, " ");;
    if (p2buf != NULL)
        *grantSize = atoi (p2buf); 
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *playout = rssfd_StringIndex (playoutArr, p2buf); 
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *start = atoi (p2buf); 
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *repeat = atoi (p2buf); 
	
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *frequency = atof (p2buf);
	
   	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " \n");
    if (p2buf != NULL)
        *power = atof (p2buf);

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the OFDMA settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_OFDMA_FFT_SIZE
/// HIFN     RSSFD_ATTR_DOCSIS31_ESPECTRUM
/// HIFN     RSSFD_ATTR_OFDMA_FIRST_ACTIVE_SUBCARRIER
/// HIFN     RSSFD_ATTR_OFDMA_LAST_ACTIVE_SUBCARRIER
/// HIFN     RSSFD_ATTR_OFDMA_CYCLIC_PREFIX
/// HIFN     RSSFD_ATTR_OFDMA_WINDOW_ROLLOFF_PERIOD
/// HIFN     RSSFD_ATTR_OFDMA_SYMBOLS_PER_FRAME
/// HIFN     RSSFD_ATTR_OFDMA_PILOT_PATTERN
/// HIFN     RSSFD_ATTR_OFDMA_CONSTELLATION
/// HIFN     RSSFD_ATTR_OFDMA_SCRAMBLER_STATE
/// HIFN     RSSFD_ATTR_OFDMA_SCRAMBLER_SEED
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     OFDMa:FFT
/// HIFN     MODulator:CHANnel<Channel>:ESPectrum
/// HIFN     OFDMa:FASubcarrier
/// HIFN     OFDMa:LASubcarrier
/// HIFN     OFDMa:CPRefix
/// HIFN     OFDMa:WINDow
/// HIFN     OFDMa:SPFrame
/// HIFN     OFDMa:PILot:PATTern
/// HIFN     OFDMa:MODulation
/// HIFN     OFDMa:SCRambler:STATe
/// HIFN     OFDMa:SCRambler:SEED
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR FFTSize/Configure the subcarrier spacing for the burst.
/// HIPAR encompassedSpectrum/Encompassed spectrum can be used as an alternative means to
/// HIPAR encompassedSpectrum/setting the number of guard subcarriers.
/// HIPAR firstActiveSubcarrier/Configure the location of the first active subcarrier.
/// HIPAR lastActiveSubcarrier/Configure the location of the last active subcarrier.
/// HIPAR cyclicPrefix/Configure the duration of the cyclic prefix.
/// HIPAR windowRolloffPeriod/Configure the duration of the rolloff period.
/// HIPAR symbolsPerFrame/Configure the number of symbols in an OFDMA frame.
/// HIPAR pilotPattern/Specify one of the seven pilot patterns defined by the DOCSIS 3.1 standard.
/// HIPAR ofdmaConstellation/Specify the modulation order for the data symbols.
/// HIPAR scramblerState/Enable or disable the data randomizer.
/// HIPAR scramblerSeed/Configure the seed to use for the data randomizer.
ViStatus _VI_FUNC rssfd_ConfigureOFDMA (
	ViSession instrSession,
	ViInt32 FFTSize,
	ViInt32 firstActiveSubcarrier,
	ViInt32 lastActiveSubcarrier,
	ViReal64 cyclicPrefix,
	ViReal64 windowRolloffPeriod,
	ViInt32 symbolsPerFrame,
	ViInt32 pilotPattern,
	ViInt32 ofdmaConstellation,
	ViBoolean scramblerState,
	ViString scramblerSeed
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_OFDMA_FFT_SIZE,
                                                 FFTSize), 2, "FFT Size");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_OFDMA_FIRST_ACTIVE_SUBCARRIER,
                                                 firstActiveSubcarrier), 4, "First Active Subcarrier");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_OFDMA_LAST_ACTIVE_SUBCARRIER,
                                                 lastActiveSubcarrier), 5, "Last Active Subcarrier");

    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_CYCLIC_PREFIX,
                                                cyclicPrefix), 6, "Cyclic Prefix");

    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_WINDOW_ROLLOFF_PERIOD,
                                                windowRolloffPeriod), 7, "Window Rolloff Period");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_OFDMA_SYMBOLS_PER_FRAME,
                                                 symbolsPerFrame), 8, "Symbols Per Frame");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_OFDMA_PILOT_PATTERN,
                                                 pilotPattern), 9, "Pilot Pattern");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_OFDMA_CONSTELLATION,
                                                 ofdmaConstellation), 10, "OFDMA Constellation");
	
    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_OFDMA_SCRAMBLER_STATE,
                                                 scramblerState), 11, "Scrambler State");

    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                 RSSFD_ATTR_OFDMA_SCRAMBLER_SEED,
                                                 scramblerSeed), 12, "Scrambler Seed");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the file to use for specifying advanced minislot profiles.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_OFDMA_ADVANCED_PROFILES
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     OFDMa:MINIslot:ADVanced
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR advancedProfiles/Set the file to use for specifying advanced minislot profiles.
ViStatus _VI_FUNC rssfd_ConfigureOFDMAAdvancedProfiles (
	ViSession instrSession,
	ViString advancedProfiles
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                RSSFD_ATTR_OFDMA_ADVANCED_PROFILES,
                                                advancedProfiles), 2, "Advanced Profiles");

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the exclusion band settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_OFDMA_EXCLUSION_START
/// HIFN     RSSFD_ATTR_OFDMA_EXCLUSION_WIDTH
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     OFDMa:EXCLusion<Exclusion>:STARt
/// HIFN     OFDMa:EXCLusion<Exclusion>:WIDTh
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR start/Subcarrier at which to start the exclusion band.
/// HIPAR width/Width of an exclusion zone from the starting subcarrier.
ViStatus _VI_FUNC rssfd_ConfigureOFDMAExclusionBands (
	ViSession instrSession,
	ViInt32 exclusion,
	ViInt32 start,
	ViInt32 width
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar  rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    if (rssfd_invalidViInt32Range (exclusion, 1, 3) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Exclusion");
    }	

    viCheckParm (rssfd_GetAttributeRepCapName (instrSession,
                                                 RSSFD_ATTR_OFDMA_EXCLUSION_START,
                                                 "Exclusion", 
                                                 exclusion - 1, 
                                                 15, 
                                                 rep_cap), 
                                                 2, "Exclusion");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, rep_cap,
                                                 RSSFD_ATTR_OFDMA_EXCLUSION_START,
                                                 start), 3, "Start");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, rep_cap,
                                                RSSFD_ATTR_OFDMA_EXCLUSION_WIDTH,
                                                width), 4, "Width");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function determines the rate at which load minislots are filled.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_OFDMA_MINISLOT_LOADING_DUTY_CYCLE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     OFDMa:DMODe:MINIslot:DCYCle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR minislotLoadingDutyCycle/Determines the rate at which load minislots are filled. Set to 0 to 
/// HIPAR minislotLoadingDutyCycle/disable simulated load carriers.
ViStatus _VI_FUNC rssfd_ConfigureOFDMADataMode(
	ViSession	instrSession,
	ViInt32	minislotLoadingDutyCycle
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_OFDMA_MINISLOT_LOADING_DUTY_CYCLE,
                                                 minislotLoadingDutyCycle), 2, "Minislot Loading Duty Cycle");
    
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the OFDMA initial ranging settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_OFDMA_IRAN_STARTING_MINISLOT
/// HIFN     RSSFD_ATTR_OFDMA_IRAN_NUMBER_OF_MINISLOTS
/// HIFN     RSSFD_ATTR_OFDMA_IRAN_NUMBER_OF_SUBCARRIERS
/// HIFN     RSSFD_ATTR_OFDMA_IRAN_MAC_ADDRESS
/// HIFN     RSSFD_ATTR_OFDMA_IRAN_DOWNSTREAM_CHANNEL
/// HIFN     RSSFD_ATTR_OFDMA_IRAN_PREAMBLE_PATTERN
/// HIFN     RSSFD_ATTR_OFDMA_IRAN_PREAMBLE_LENGTH
/// HIFN     RSSFD_ATTR_OFDMA_IRAN_PREAMBLE_VALUE_OFFSET
/// HIFN     RSSFD_ATTR_OFDMA_IRAN_PREAMBLE_FILL_DUTY_CYCLE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     OFDMa:IRANging:MINIslot:STARt
/// HIFN     OFDMa:IRANging:MINIslot:COUNt
/// HIFN     OFDMa:IRANging:SUBCarrier:COUNt
/// HIFN     OFDMa:IRANanging:MAC
/// HIFN     OFDMa:IRANging:CHANnel
/// HIFN     OFDMa:IRANanging:PREAmble:PATTern
/// HIFN     OFDMa:IRANging:PREAmble:LENGth
/// HIFN     OFDMa:IRANging:PREAmble:OFFSet
/// HIFN     OFDMa:IRANging:MINIslot:DCYCle
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR startingMinislot/Specify which minislot is the start of a particular frame.
/// HIPAR numberOfMinislots/Specify the number of minislots to use.
/// HIPAR numberOfSubcarriers/Specify the number of subcarriers to use.
/// HIPAR macAddress/Specify the mac address used for the data payload.
/// HIPAR downstreamChannel/Specify the associated downstream channel used for the data payload.
/// HIPAR preamblePattern/The preamble pattern for the data, if nothing is specified the
/// HIPAR preamblePattern/default preamble pattern will be used.       
/// HIPAR preambleLength/Specify the preamble length.
/// HIPAR preambleValueOffset/The starting offset of the preamble pattern.
/// HIPAR fillRemainingMinislots/Configure if the remaining minislots to not allocated to the
/// HIPAR fillRemainingMinislots/target user are filled with data or zero valued.
ViStatus _VI_FUNC rssfd_ConfigureOFDMAInitialRanging (
	ViSession instrSession,
	ViInt32 startingMinislot,
	ViInt32 numberOfMinislots,
	ViInt32 numberOfSubcarriers,
	ViString macAddress,
	ViInt32 downstreamChannel,
	ViString preamblePattern,	
	ViInt32 preambleLength,
	ViInt32 preambleValueOffset,
	ViInt32 minislotLoadingDutyCycle	
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));


    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_IRAN_STARTING_MINISLOT,
                                                startingMinislot), 2, "Starting Minislot");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_IRAN_NUMBER_OF_MINISLOTS,
                                                numberOfMinislots), 3, "Number Of Minislots");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_IRAN_NUMBER_OF_SUBCARRIERS,
                                                numberOfSubcarriers), 4, "Number Of Subcarriers");

    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                RSSFD_ATTR_OFDMA_IRAN_MAC_ADDRESS,
                                                macAddress), 5, "MAC Address");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_IRAN_DOWNSTREAM_CHANNEL,
                                                downstreamChannel), 6, "Downstream Channel");
	
    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                RSSFD_ATTR_OFDMA_IRAN_PREAMBLE_PATTERN,
                                                preamblePattern), 7, "Preamble Pattern");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_IRAN_PREAMBLE_LENGTH,
                                                preambleLength), 8, "Preamble Length");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_IRAN_PREAMBLE_VALUE_OFFSET,
                                                preambleValueOffset), 9, "Preamble Value Offset");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_OFDMA_IRAN_MINISLOT_LOADING_DUTY_CYCLE,
                                                 minislotLoadingDutyCycle), 10, "Minislot Loading Duty Cycle");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the OFDMA initial ranging settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_OFDMA_FRAN_NUMBER_OF_FRAMES
/// HIFN     RSSFD_ATTR_OFDMA_FRAN_STARTING_MINISLOT
/// HIFN     RSSFD_ATTR_OFDMA_FRAN_NUMBER_OF_MINISLOTS
/// HIFN     RSSFD_ATTR_OFDMA_FRAN_NUMBER_OF_SUBCARRIERS
/// HIFN     RSSFD_ATTR_OFDMA_FRAN_PREAMBLE_PATTERN
/// HIFN     RSSFD_ATTR_OFDMA_FRAN_PREAMBLE_VALUE_OFFSET
/// HIFN     RSSFD_ATTR_OFDMA_FRAN_DATA_FILENAME
/// HIFN     RSSFD_ATTR_OFDMA_FRAN_MINISLOT_LOADING_DUTY_CYCLE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     OFDMa:FRANging:FRAMe:COUNt
/// HIFN     OFDMa:FRANanging:MINIslot:STARt
/// HIFN     OFDMa:FRANging:MINIslot:COUNt
/// HIFN     OFDMa:FRANging:SUBCarrier:COUNt
/// HIFN     OFDMa:FRANging:PREAmble:PATTern
/// HIFN     OFDMa:FRANging:PREAmble:OFFSet
/// HIFN     OFDMa:FRANging:DATA
/// HIFN     OFDMa:FRANging:MINIslot:DCYCle
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberOfFrames/Specify the number of frames to generate.
/// HIPAR startingMinislot/Specify which minislot is the start of a particular frame.
/// HIPAR numberOfMinislots/Specify the number of minislots to use.
/// HIPAR numberOfSubcarriers/Specify the number of subcarriers to use.
/// HIPAR preamblePattern/The preamble pattern for the data, if nothing is specified the
/// HIPAR preamblePattern/default preamble pattern will be used.
/// HIPAR preambleValueOffset/The starting offset of the preamble pattern.
/// HIPAR dataFilename/Set the filename of the file containing the binary data to send
/// HIPAR dataFilename/in the burst.
/// HIPAR fillRemainingMinislots/Configure if the remaining minislots to not allocated to the
/// HIPAR fillRemainingMinislots/target user are filled with data or zero valued.
ViStatus _VI_FUNC rssfd_ConfigureOFDMAFineRanging (
	ViSession instrSession,
	ViInt32 startingMinislot,
	ViInt32 numberOfMinislots,
	ViInt32 numberOfSubcarriers,
	ViString preamblePattern,
	ViInt32 preambleValueOffset,
	ViString dataFilename,	
	ViInt32 minislotLoadingDutyCycle
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));


    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_FRAN_STARTING_MINISLOT,
                                                startingMinislot), 3, "Starting Minislot");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_FRAN_NUMBER_OF_MINISLOTS,
                                                numberOfMinislots), 4, "Number Of Minislots");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_FRAN_NUMBER_OF_SUBCARRIERS,
                                                numberOfSubcarriers), 5, "Number Of Subcarriers");

    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                RSSFD_ATTR_OFDMA_FRAN_PREAMBLE_PATTERN,
                                                preamblePattern), 6, "Preamble Pattern");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_FRAN_PREAMBLE_VALUE_OFFSET,
                                                preambleValueOffset), 7, "Preamble Value Offset");

    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                RSSFD_ATTR_OFDMA_FRAN_DATA_FILENAME,
                                                dataFilename), 8, "Data Filename");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_OFDMA_FRAN_MINISLOT_LOADING_DUTY_CYCLE,
                                                 minislotLoadingDutyCycle), 9, "Minislot Loading Duty Cycle");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the OFDMA initial ranging settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_OFDMA_WPR_SUBCARRIER_START
/// HIFN     RSSFD_ATTR_OFDMA_WPR_SUBCARRIER_SKIP
/// HIFN     RSSFD_ATTR_OFDMA_WPR_STAGGER
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     OFDMa:WPRobe:FRAMe:COUNt
/// HIFN     OFDMa:WPRobe:SUBCarrier:STARt
/// HIFN     OFDMa:WPRobe:SUBCarrier:SKIP
/// HIFN     OFDMa:WPRobe:STAGger
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR subcarrierStart/Specify the start subcarrier.
/// HIPAR subcarrierSkip/Specify the subcarrier skip.
/// HIPAR stagger/Enable or disable the stagger option.
ViStatus _VI_FUNC rssfd_ConfigureOFDMAWidebandProbe (
	ViSession instrSession,
	ViInt32 subcarrierStart,
	ViInt32 subcarrierSkip,
	ViInt32 symbolInFrame,	
	ViBoolean stagger,
	ViInt32 minislotLoadingDutyCycle
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_WPR_SUBCARRIER_START,
                                                subcarrierStart), 2, "Subcarrier Start");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_WPR_SUBCARRIER_SKIP,
                                                subcarrierSkip), 3, "Subcarrier Skip");

	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_WPR_SUBCARRIER_SYMBOL_IN_FRAME,
                                                symbolInFrame), 4, "Symbol in Frame");
		
    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_OFDMA_WPR_STAGGER,
	                                                 stagger), 5, "Stagger");
	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_WPR_MINISLOT_LOADING_DUTY_CYCLE,
                                                minislotLoadingDutyCycle), 6, "Minislot Loading Duty Cycle");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the OFDMA initial ranging settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     
/// HIFN     RSSFD_ATTR_OFDMA_BRM_MINISLOT_IN_FRAME
/// HIFN     RSSFD_ATTR_OFDMA_BRM_SUBSLOT
/// HIFN     RSSFD_ATTR_OFDMA_BRM_NUMBER_OF_BYTES
/// HIFN     RSSFD_ATTR_OFDMA_BRM_SID
/// HIFN     RSSFD_ATTR_OFDMA_BRM_MINISLOT_LOADING_DUTY_CYCLE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     OFDMa:BRMessage:FRAMe:MINIslot
/// HIFN     OFDMa:BRMessage:SUBSlot
/// HIFN     OFDMa:BRMessage:MINIslot:NUMBer
/// HIFN     OFDMa:BRMessage:SID
/// HIFN     OFDMa:BRMessage:MINIslot:DCYCle
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR minislotInFrame/Configure which minislot in the frame will be used for the BW request.
/// HIPAR subslot/Configure the subslot within the frame for transmitting the BW request.
/// HIPAR numberOfBytes/Configure the number of bytes requested.
/// HIPAR SID/Configure the SID to be used for the request.
/// HIPAR fillRemainingMinislots/Configure if the remaining minislots to not allocated to the
/// HIPAR fillRemainingMinislots/target user are filled with data or zero valued.
ViStatus _VI_FUNC rssfd_ConfigureOFDMABandwidthRequestedMessage (
	ViSession instrSession,
	ViInt32 minislotInFrame,
	ViInt32 subslot,
	ViInt32 numberOfBytes,
	ViString SID,
	ViInt32 minislotLoadingDutyCycle 
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));


    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_BRM_MINISLOT_IN_FRAME,
                                                minislotInFrame), 2, "Minislot In Frame");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_BRM_SUBSLOT,
                                                subslot), 3, "Subslot");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_OFDMA_BRM_NUMBER_OF_BYTES,
												numberOfBytes), 4, "Number Of Bytes");
	
    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                 RSSFD_ATTR_OFDMA_BRM_SID,
                                                 SID), 5, "SID");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_OFDMA_BRM_MINISLOT_LOADING_DUTY_CYCLE,
                                                 minislotLoadingDutyCycle), 6, "Minislot Loading Duty Cycle");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the upstream ARB channel settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_UPSTREAM_ARB_TRANSMIT
/// HIFN     RSSFD_ATTR_UPSTREAM_ARB_FREQUENCY
/// HIFN     RSSFD_ATTR_UPSTREAM_ARB_POWER
/// HIFN     RSSFD_ATTR_UPSTREAM_ARB_TRIGGER_MODE
/// HIFN     RSSFD_ATTR_UPSTREAM_ARB_OUTPUT_DELAY
/// HIFN     RSSFD_ATTR_UPSTREAM_ARB_INTER_BURST_GAP
/// HIFN     RSSFD_ATTR_UPSTREAM_ARB_WAVEFORM
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MODulator:CHANnel:UPstream:ARBitrary:STATe
/// HIFN     FREQuency:CHANnel:UPstream:ARBitrary:FREQuency
/// HIFN     POWer:CHANnel:UPstream:ARBitrary:LEVel
/// HIFN     MODulator:CHANnel:UPstream:ARBitrary:MODE
/// HIFN     MODulator:CHANnel:UPstream:ARBitrary:DELay
/// HIFN     MODulator:CHANnel:UPstream:ARBitrary:IBGAp
/// HIFN     MODulator:CHANnel:UPstream:ARBitrary:WAVeform:SELect
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transmit/Switches the modulator on or off for a specified channel.
/// HIPAR frequency/Sets the frequency for a specified channel.
/// HIPAR power/Sets the power level for a specified channel.
/// HIPAR triggerMode/Specify whether or not the trigger is used to initiate playout of the 
/// HIPAR triggerMode/specific arb channel.
/// HIPAR outputDelay/Specify the amount of time between receiving trigger event and playing 
/// HIPAR outputDelay/out the arb waveform. Only used when the channel trigger mode is set to 
/// HIPAR outputDelay/triggered.
/// HIPAR interBurstGap/The amount of time between waveform playouts. This is only used in 
/// HIPAR interBurstGap/triggered mode. When the inter-burst gap is larger than 9999 us, the 
/// HIPAR interBurstGap/playout will only occur once, otherwise it will be periodic with the 
/// HIPAR interBurstGap/inter-burst gap.
/// HIPAR arbFile/Selects and loads an arbitrary waveform file for a upstream channel.
ViStatus _VI_FUNC rssfd_ConfigureUpstreamArbitraryWaveform(
	ViSession	instrSession,
	ViBoolean	transmit,
	ViReal64	frequency,
	ViReal64	power,
	ViInt32	triggerMode,
	ViReal64	outputDelay,
	ViInt32 	interBurstGap,
	ViString	arbFile
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    
    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_UPSTREAM_ARB_TRANSMIT,
                                                 transmit), 2, "Transmit");
	
    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_UPSTREAM_ARB_FREQUENCY,
                                                frequency), 3, "Frequency");
	    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_UPSTREAM_ARB_POWER,
                                                power), 4, "Power");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_UPSTREAM_ARB_TRIGGER_MODE,
                                               triggerMode), 5, "Trigger Mode");
	
    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_UPSTREAM_ARB_OUTPUT_DELAY,
                                                outputDelay), 6, "Output Delay");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_UPSTREAM_ARB_INTER_BURST_GAP,
                                                interBurstGap), 7, "Inter Burst Gap");
	
    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                               RSSFD_ATTR_UPSTREAM_ARB_WAVEFORM,
                                               arbFile), 8, "ARB File");

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function  responds with a list of channel parameter settings in 
/// HIFN  this order:
/// HIFN      STAT Output state
/// HIFN      FREQ Frequency
/// HIFN      POW Power
/// HIFN      MODE Playout mode
/// HIFN      DEL Trigger output delay
/// HIFN      IBG Inter-burst gap
/// HIFN      SEL AWG file selection
/// HIFN     
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     READ:CHANnel:UPstream:ARBitrary:CONFig?
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frequency/Returns the frequency.
/// HIPAR power/Returns the power.
/// HIPAR triggerMode/Returns the playout mode.
/// HIPAR outputDelay/Returns the output playout delay.
/// HIPAR transmit/Returns the output state.
/// HIPAR interBurstGap/Returns the inter-burst gap.
/// HIPAR arbFile/Returns the AWG file selection.
ViStatus _VI_FUNC rssfd_QueryUpstreamARBConfiguration(
	ViSession	instrSession,
	ViBoolean*	transmit, 
	ViReal64*	frequency,
	ViReal64*	power,
	ViInt32*	triggerMode,
	ViReal64*	outputDelay,
	ViInt32*	interBurstGap,
	ViChar	arbFile[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar*     ext_buf=VI_NULL;
    ViChar*     p2buf;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

   
    checkErr (viPrintf (instrSession, "READ:CHAN:UP:ARB:CONF?\n"));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, NULL));

	
	  //STAT ON FREQ 4.000000e7   POW 30.0 MODE CONT DEL 0 IBG 0 SEL /Factory/waveform/off.wv
    
	p2buf = strtok (ext_buf, " ");

    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *transmit =  rssfd_StringIndex (offOnArr, p2buf) == VI_TRUE;
  
  	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *frequency = atof (p2buf);
  
    p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *power = atof (p2buf);
   
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *triggerMode =  rssfd_StringIndex (triggerModeArr, p2buf);
   
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
    if (p2buf != NULL)
        *outputDelay = atof (p2buf);
    
	p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " ");
	if (p2buf != NULL)
		*interBurstGap = atoi (p2buf);
	 
    p2buf = strtok (NULL, " ");
    p2buf = strtok (NULL, " \n");
	if (p2buf != NULL)
        strcpy (arbFile, p2buf); 

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    

	return error;
}




/// HIFN  This function returns the upstream phase noise level.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     NOISe:PHASenoise:UPstream:AT?
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frequency/Frequency point in the noise mask.
/// HIPAR level/Returns the upstream phase noise level for selected frequency.
ViStatus _VI_FUNC rssfd_QueryPhaseNoiseLevelUpstream (
	ViSession instrSession,
		ViReal64 frequency,
	ViReal64 *level
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 

    viCheckErr (viQueryf (instrSession, "NOIS:PHAS:UP:AT? %lf\n", "%lf", frequency, level));
	
    checkErr( rssfd_CheckStatus (instrSession)); 
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the HUM settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_HUM_STATE
/// HIFN     RSSFD_ATTR_HUM_FREQUENCY
/// HIFN     RSSFD_ATTR_HUM_DEPTH
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     HUM:STATe
/// HIFN     HUM:FREQuency
/// HIFN     HUM:DEPth
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Switches the impairment on or off.
/// HIPAR frequency/Sets the frequency for AC hum.
/// HIPAR depth/Sets the AC hum level for the unit.
ViStatus _VI_FUNC rssfd_ConfigureHUM(
	ViSession	instrSession,
	ViBoolean	state,
	ViReal64	frequency,
	ViReal64	depth
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_HUM_STATE,
                                                 state), 2, "State");

    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_HUM_FREQUENCY,
                                                frequency), 3, "Frequency");

    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_HUM_DEPTH,
                                                depth), 4, "Depth");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the AWGN noise settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_NOISE_AWGN_MODE
/// HIFN     RSSFD_ATTR_NOISE_AWGN_BANDWIDTH
/// HIFN     RSSFD_ATTR_NOISE_AWGN_AMPLITUDE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     NOISe:AWGN:MODE
/// HIFN     NOISe:AWGN:BANDwidth
/// HIFN     NOISe:AWGN:AMPlitude
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR awgnMode/Switches the noise off or places it in a locked or free mode.
/// HIPAR awgnChannelType/This parameter indicates the type of the channel that the noise is 
/// HIPAR awgnChannelType/associated with.
/// HIPAR awgnChannel/Sets the channel where the noise will be centered. 
/// HIPAR awgnFrequency/Sets the frequency position where the noise will be centered. 
/// HIPAR awgnBandwidth/Sets the spectral width of the noise.
/// HIPAR awgnAmplitude/Sets the magnitude of the noise.
ViStatus _VI_FUNC rssfd_ConfigureNoiseAWGN(
	ViSession	instrSession,
	ViBoolean	awgnMode,
	ViReal64	awgnBandwidth,
	ViInt32		awgnCNReference, 
	ViReal64	awgnAmplitude,
	ViReal64	awgnReceiverBandwidth
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                               RSSFD_ATTR_NOISE_AWGN_MODE,
                                               awgnMode), 2, "AWGN Mode");

	viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_NOISE_AWGN_BANDWIDTH,
                                                awgnBandwidth), 3, "AWGN Bandwidth");
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_NOISE_AWGN_CN_REFERENCE,
                                               awgnCNReference), 4, "AWGN C/N Reference");	
	viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_NOISE_AWGN_AMPLITUDE,
                                                awgnAmplitude), 5, "AWGN Amplitude");
	

	viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "", 
                                                RSSFD_ATTR_NOISE_AWGN_RECEIVER_BANDWIDTH,
                                                awgnReceiverBandwidth), 6, "AWGN Receiver Bandwidth");  
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_BER_MODE
/// HIFN     RSSFD_ATTR_BER_VALUE
/// HIFN     
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     BERate:MODE
/// HIFN     BERate:VALue
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR berMode/Switches the BER off or on.
/// HIPAR berValue/Sets the value of the bit error rate to use.
ViStatus _VI_FUNC rssfd_ConfigureBitErrorRate(
	ViSession	instrSession,
	ViBoolean	berMode,
	ViReal64	berValue
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_BER_MODE,
                                               berMode), 2, "BER Mode");

	viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_BER_VALUE,
                                                berValue), 3, "BER Value");

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}
/// HIFN  This function configures the phase noise settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_PHASE_NOISE_STATE
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     NOISe:PHASenoise:STATe
/// HIFN     NOISe:PHASenoise:AT
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Sets the state of the phase noise generation.
/// HIPAR frequency/Sets the frequency point in the noise mask.
/// HIPAR level/Sets the noise power level in dBc.
ViStatus _VI_FUNC rssfd_ConfigurePhaseNoise (
	ViSession instrSession,
	ViBoolean state, 
	ViReal64 frequency,
	ViReal64 level
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViBoolean (instrSession, "",
                                                 RSSFD_ATTR_PHASE_NOISE_STATE,
                                                 state), 2, "State");

	viCheckErr (viPrintf (instrSession, "NOIS:PHAS:AT %lf,%lf\n", frequency, level));
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the phase noise level.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     NOISe:PHASenoise:AT?
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frequency/Frequency point in the noise mask.
/// HIPAR level/Returns the phase noise level for selected frequency.
ViStatus _VI_FUNC rssfd_QueryPhaseNoiseLevel (
	ViSession instrSession,
	ViReal64 frequency,
	ViReal64 *level
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 

    viCheckErr (viQueryf (instrSession, "NOIS:PHAS:AT? %lf\n", "%lf", frequency, level));
	
    checkErr( rssfd_CheckStatus (instrSession)); 
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the downstream timing settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_DOWNSTREAM_TIMING_DOCSIS_TIMESTAMP
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     TIMing:DOWNstream:TSTamp
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR docsisTimestamp/Used to set the downstream DOCSIS 3.1 timestamp seed value. This value 
/// HIPAR docsisTimestamp/will be latched to the internal DOCSIS clock at the time of the trigger 
/// HIPAR docsisTimestamp/in event. 
ViStatus _VI_FUNC rssfd_ConfigureDownstreamTimingSetting(
	ViSession	instrSession,
	ViInt32	docsisTimestamp
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_DOWNSTREAM_TIMING_DOCSIS_TIMESTAMP,
                                               docsisTimestamp), 2, "DOCSIS Timestamp");

    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the upstream timing settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_UPSTREAM_TIMING_DOCSIS_EXTENDED_TIMESTAMP
/// HIFN     RSSFD_ATTR_UPSTREAM_TIMING_TX_TIME_OFFSET_ADJUST
/// HIFN     RSSFD_ATTR_UPSTREAM_TIMING_DOCSIS_TIMESTAMP
/// HIFN     RSSFD_ATTR_UPSTREAM_TIMING_MINISLOT_COUNTER
/// HIFN     RSSFD_ATTR_UPSTREAM_TIMING_FRAME_COUNTER
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     TIMing:UPstream:TSTamp:EXTended
/// HIFN     TIMing:UPstream:TSTamp:OFFSet
/// HIFN     TIMing:UPstream:TSTamp
/// HIFN     TIMing:UPstream:TSTamp:MINI
/// HIFN     TIMing:UPstream:TSTamp:FRAMe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR txTimeOffsetAdjust/The offset in time to apply to the DOCSIS timestamp when synchronizing 
/// HIPAR txTimeOffsetAdjust/to an actual  point in time.
/// HIPAR docsisTimestamp/The upstream timestamp parameter is used to associate a specific 
/// HIPAR docsisTimestamp/DOCSIS timestamp to a particular point in time. The timestamp value can 
/// HIPAR docsisTimestamp/be specified as either a 32 bit DOCSIS timestamp or a 64 bit extended 
/// HIPAR docsisTimestamp/DOCSIS timestamp. A separate SCPI command is used to indicate which 
/// HIPAR docsisTimestamp/format is to be used.
/// HIPAR minislotCounter/Configure the minislot associated to the specified upstream timestamp 
/// HIPAR minislotCounter/value. Once the device is synchronized in time, this parameter can be 
/// HIPAR minislotCounter/used to associate minislot numbering to a definite point in time. 
/// HIPAR frameCounter/Configure the frame associated to the specified upstream timestamp 
/// HIPAR frameCounter/value. This is used for SCDMA and OFDMA burst types. For ATDMA, the 
/// HIPAR frameCounter/value should match the minislot value as in this case each minislot can 
/// HIPAR frameCounter/be assumed to occupy one frame. 
/// HIPAR usedDOCSISExtendedTimestamp/The upstream timestamp parameter is used to associate a specific 
/// HIPAR usedDOCSISExtendedTimestamp/DOCSIS timestamp to a particular point in time. The timestamp value can 
/// HIPAR usedDOCSISExtendedTimestamp/be specified as either a 32 bit DOCSIS timestamp or a 64 bit extended 
/// HIPAR usedDOCSISExtendedTimestamp/DOCSIS timestamp. A separate SCPI command is used to indicate which 
/// HIPAR usedDOCSISExtendedTimestamp/format is to be used. 
ViStatus _VI_FUNC rssfd_ConfigureUpstreamTimingSettings(
	ViSession	instrSession,
	ViInt32	usedDOCSISExtendedTimestamp,
	ViInt32	txTimeOffsetAdjust,
	ViInt32	docsisTimestamp,
	ViInt32	minislotCounter,
	ViInt32	frameCounter
	
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_UPSTREAM_TIMING_DOCSIS_EXTENDED_TIMESTAMP,
                                                 usedDOCSISExtendedTimestamp), 2, "Used DOCSIS Extended Timestamp");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_UPSTREAM_TIMING_TX_TIME_OFFSET_ADJUST,
                                                txTimeOffsetAdjust), 3, "TX Time Offset Adjust");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_UPSTREAM_TIMING_DOCSIS_TIMESTAMP,
                                                docsisTimestamp), 4, "DOCSIS Timestamp");

	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_UPSTREAM_TIMING_MINISLOT_COUNTER,
                                                minislotCounter), 5, "Minislot Counter");

	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_UPSTREAM_TIMING_FRAME_COUNTER,
                                                frameCounter), 6, "Frame Counter");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the trigger settings.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_TRIGGER_SOURCE
/// HIFN     RSSFD_ATTR_TRIGGER_LEVEL
/// HIFN     RSSFD_ATTR_TRIGGER_EDGE
/// HIFN     RSSFD_ATTR_TRIGGER_DELAY
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger:SOURce
/// HIFN     TRIGger:LEVel
/// HIFN     TRIGger:EDGe
/// HIFN     TRIGger:OUTPut:DELay
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerSource/Configure where the trigger source is activated
/// HIPAR triggerLevel/Configure the output state when a trigger occurs.
/// HIPAR edge/Configure the trigger event to activate on either a falling or
/// HIPAR edge/rising edge of external source.
/// HIPAR delay/Configure the amount of time between the start of a trigger
/// HIPAR delay/event and the transmission of the output trigger on the external
/// HIPAR delay/trigger out port found on the back of the device.
ViStatus _VI_FUNC rssfd_ConfigureTrigger (ViSession instrSession,
                                          ViInt32 triggerSource,
                                          ViInt32 triggerInSensitivity,
                                          ViReal64 markerOutStartDelay,
                                          ViInt32 markerOutPolarity
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                 RSSFD_ATTR_TRIGGER_SOURCE,
                                                 triggerSource), 2, "Trigger Source");

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_TRIGGER_IN_SENSITIVITY,
                                                triggerInSensitivity), 3, "Trigger In Sensitivity");
	
    viCheckParm (rssfd_SetAttributeViReal64 (instrSession, "",
                                                RSSFD_ATTR_MARKER_OUT_START_DELAY,
                                                markerOutStartDelay), 4, "Marker Out Start Delay");
	viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                                RSSFD_ATTR_MARKER_OUT_POLARITY,
                                                markerOutPolarity), 5, "Marker Out Polarity");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function allows for the remote interface to simulate an external 
/// HIFN  trigger similar to pressing the manual trigger button on the web user 
/// HIFN  interface. The trigger source must be set to internal to utilize this 
/// HIFN  command. 
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_TRIGGER_IMMEDIATE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger:INITiate:IMMediate
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rssfd_ConfigureTriggerImmediate( ViSession	instrSession)

{
	ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    viCheckErr (rssfd_SetAttributeViString (instrSession, "",
                                               RSSFD_ATTR_TRIGGER_IMMEDIATE, NULL))
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the command communication.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_IP_ADDRESS_COMMAND
/// HIFN     RSSFD_ATTR_SUBNET_MASK
/// HIFN     RSSFD_ATTR_GATEWAY
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:COMMunication:NETWork:IPADdress:COMMand
/// HIFN     SYSTem:COMMunication:NETWork:SUBNet
/// HIFN     SYSTem:COMMunication:NETWork:GATeway
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rssfd_init or 
/// HIPAR instrSession/rssfd_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR ipAddress/This sets the IP address on the 10/100 Mbs LAN port
/// HIPAR subnetMask/This sets the mask for the IP address allowing the user to set the 
/// HIPAR subnetMask/class of the network to comply with that of the network to which the 
/// HIPAR subnetMask/CLG is connected.
/// HIPAR gateway/This allows the user to identify the gateway the SFD is to use for 
/// HIPAR gateway/the network.
ViStatus _VI_FUNC rssfd_ConfigureCommandCommunication(
	ViSession	instrSession,
	ViString	ipAddress,
	ViString	subnetMask,
	ViString	gateway
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                RSSFD_ATTR_IP_ADDRESS_COMMAND,
                                                ipAddress), 2, "IP Address");

    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                RSSFD_ATTR_SUBNET_MASK,
                                                subnetMask), 3, "Subnet Mask");

    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                                RSSFD_ATTR_GATEWAY,
                                                gateway), 4, "Gateway");
	
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the SFD for a static IP address or for DHCP 
/// HIFN  where there must be a DHCP server on the network to assign a unique IP 
/// HIFN  address to each device on the network.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_NETWORK_TYPE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:COMMunication:NETWork:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rssfd_init or 
/// HIPAR instrSession/rssfd_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR networkType/The user can configure the SFD for a static IP address or for DHCP 
/// HIPAR networkType/where there must be a DHCP server on the network to assign a unique IP 
/// HIPAR networkType/address to each device on the network.
ViStatus _VI_FUNC rssfd_ConfigureNetworkType(
	ViSession	instrSession,
	ViInt32		networkType
)
{
    ViStatus  error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_NETWORK_TYPE,
                                               networkType), 2, "Network Type");
    
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the data communication SFP.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_SFP_DATA_IP_ADDRESS
/// HIFN     RSSFD_ATTR_SFP_DATA_RATE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:COMMunicate:NETWork:SFP:IPADdress
/// HIFN     SYSTem:COMMunicate:NETWork:SFP:RATE
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rssfd_init or 
/// HIPAR instrSession/rssfd_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR IPAddress/Configure the IP address used by the SFP ports on the back of the unit.
/// HIPAR dataRate/Configure the input data rate for the SFP ports on the unit.
ViStatus _VI_FUNC rssfd_ConfigureSFPPort (
	ViSession instrSession,
	ViString IPAddress,
	ViInt32 dataRate
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

   
	
    viCheckParm (rssfd_SetAttributeViString (instrSession, "",
                                               RSSFD_ATTR_SFP_DATA_IP_ADDRESS,
                                               IPAddress), 3, "IP Address");
	
    viCheckParm (rssfd_SetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_SFP_DATA_RATE,
                                               dataRate), 4, "Data Rate");
    
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  The MMEMory (mass memory) subsystem function provides commands which 
/// HIFN  allow for access to the storage media of the instrument and for storing 
/// HIFN  and loading various instrument settings.
/// HIFN    
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_FILE_MANAGER_MAKE_DIR
/// HIFN     -
/// HIFN     -
/// HIFN     RSSFD_ATTR_FILE_MANAGER_DELETE
/// HIFN     RSSFD_ATTR_FILE_MANAGER_DELETE_DIR
/// HIFN     RSSFD_ATTR_FILE_MANAGER_EDIT_PATH
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MMEMory:MDIRectory <file_name>
/// HIFN     MMEMory:COPY <file_source>,<file_destination>
/// HIFN     MMEMory:MOVE <file_source>,<file_destination>
/// HIFN     MMEMory:DELete
/// HIFN     MMEMory:RDIRectory
/// HIFN     MMEMory:CDIRectory
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR source/This control specifies the source of the operation to be performed. If 
/// HIPAR source/the operation requires a single parameter, this control specifies that 
/// HIPAR source/parameter.
/// HIPAR destination/This control specifies the destination of the operation to be 
/// HIPAR destination/performed.
ViStatus _VI_FUNC rssfd_FileManagerOperations(
	ViSession	instrSession,
	ViInt32		operation,
	ViString	source,
	ViString	destination
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   	buffer[RSSFD_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    switch (operation)
    {
        case RSSFD_VAL_FILE_COPY:
            sprintf (buffer, "MMEM:COPY '%s','%s'\n", source, destination);
            checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
            checkErr (rssfd_CheckStatus (instrSession));
            break;
        case RSSFD_VAL_FILE_RENAME:
            sprintf (buffer, "MMEM:MOVE '%s','%s'\n", source, destination);
            checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
            checkErr (rssfd_CheckStatus (instrSession));
            break;
        case RSSFD_VAL_FILE_DELETE:
            viCheckParm (rssfd_SetAttributeViString(instrSession, "", RSSFD_ATTR_FILE_MANAGER_DELETE, source), 3, "Source");
            break;
        case RSSFD_VAL_FILE_NEW:
            viCheckParm (rssfd_SetAttributeViString(instrSession, "", RSSFD_ATTR_FILE_MANAGER_MAKE_DIR, source), 3, "Source");
            break;
        case RSSFD_VAL_FILE_CDIR:
            viCheckParm (rssfd_SetAttributeViString(instrSession, "", RSSFD_ATTR_FILE_MANAGER_EDIT_PATH, source), 3, "Source");
            break;
        case RSSFD_VAL_FILE_RDIR:
            viCheckParm (rssfd_SetAttributeViString(instrSession, "", RSSFD_ATTR_FILE_MANAGER_DELETE_DIR, source), 3, "Source");
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operation");
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;}


/// HIFN  This function reads out the number of files in the current directory.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_FILE_MANAGER_CATALOG_LENGTH
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MMEMory:CATalog:LENGth?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberofFiles/Returns the number of files of the current directory.
ViStatus _VI_FUNC rssfd_FileCount(
	ViSession	instrSession,
	ViInt32*	numberofFiles
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_GetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_FILE_MANAGER_CATALOG_LENGTH,
                                               numberofFiles), 2, "Number of Files");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function reads the indicated directory. According to DOS 
/// HIFN  convention, wild card characters can be entered in order to query eg. a 
/// HIFN  list of all file of a certain type.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MMEMory:CATalog?
/// HIFN     *CLS
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufferSize/Output parameter.  
/// HIPAR output/This indicator returns the files in the indicated directory as comma 
/// HIPAR output/separated strings, for example: 'SPOOL1.WMF','SPOOL2.WMF','SPOOL3.WMF'
ViStatus _VI_FUNC rssfd_FileDirectory(
	ViSession	instrSession,
	ViInt32		bufferSize,
	ViChar  	output[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   	buffer[RSSFD_IO_BUFFER_SIZE] = "";
    ViChar   	*p2buf = NULL;
    ViUInt32  	count = 0;
    ViUInt32  	length = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckAlloc (output);
    
    sprintf (buffer, "MMEM:CAT?\n");
    
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2buf, &count));
    
    length = ((ViInt32)count > bufferSize) ? bufferSize : count;
    memcpy (output, p2buf, length);
    
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  Returns the number of subdirectories of the current directory.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSSFD_ATTR_FILE_MANAGER_DIRECTORY_CATALOG_LENGTH
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MMEMory:DCATalog:LENGth?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberofSubdirectories/Returns the number of files and subdirectories of the current 
/// HIPAR numberofSubdirectories/directory.
ViStatus _VI_FUNC rssfd_FileSubdirectoriesCount(
	ViSession	instrSession,
	ViInt32*	numberofSubdirectories
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfd_GetAttributeViInt32 (instrSession, "",
                                               RSSFD_ATTR_FILE_MANAGER_DIRECTORY_CATALOG_LENGTH,
                                               numberofSubdirectories), 2, "Number of Subdirectories");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function reads the names of subdirectories.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     MMEMory:DCATalog?
/// HIFN     *CLS
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be 
/// HIPAR directory/performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufferSize/Output parameter.  
/// HIPAR output/This indicator returns the names of subdirectories.
ViStatus _VI_FUNC rssfd_FileSubdirectories(
	ViSession	instrSession,
	ViInt32		bufferSize,
	ViChar		output[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   	buffer[RSSFD_IO_BUFFER_SIZE] = "";
    ViChar   	*p2buf = NULL;
    ViUInt32  	count = 0;
    ViUInt32  	length = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckAlloc (output);
    
    sprintf (buffer, "MMEM:DCAT?\n");
    
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2buf, &count));
    
    length = ((ViInt32)count > bufferSize) ? bufferSize : count;
    memcpy (output, p2buf, length);
    
    checkErr (rssfd_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function saves or loads the unit settings to or from file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR stateNumber/Defines save and recall state position.
/// HIPAR path/This control specifies the source of the operation to be performed. 
ViStatus _VI_FUNC rssfd_SaveLoadSettings (
	ViSession instrSession,
	ViInt32 operation,
	ViInt32 stateNumber, 
	ViString path
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer[RSSFD_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    switch (operation)
    {
        case RSSFD_VAL_MEM_SAVE:
            sprintf (buffer, "MMEM:STOR:STAT %ld,'%s'\n", stateNumber, path);
            checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
            checkErr (rssfd_CheckStatus (instrSession));
            break;
        case RSSFD_VAL_MEM_RECALL:
            sprintf (buffer, "MMEM:LOAD:STAT %ld,'%s'\n", stateNumber, path);
            checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
            checkErr (rssfd_CheckStatus (instrSession));
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operation");
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the Enable status questionable and operating 
/// HIFN  registers.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     (1) For detailed description of Status Reporting System see also 
/// HIFN  operating manual.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     STATus:PRESet
/// HIFN     STATus:QUEStionable:ENABle
/// HIFN     STATus:OPERation:ENABle
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RSSFD_init or 
/// HIPAR instrSession/RSSFD_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR registerOperation/Specifies the operation to perform.
/// HIPAR enable/This control sets the enable bits of the status register.  
ViStatus _VI_FUNC rssfd_SetStatusRegister(
	ViSession	instrSession,
	ViInt32		registerOperation,
	ViInt32		enable
)
{
    ViStatus  error = VI_SUCCESS;                      

	checkErr( Rs_LockSession (instrSession, VI_NULL)); 
	
    viCheckParm (rssfd_invalidViInt32Range (registerOperation, 0, 2), 2, "Register Operation"); 
	
    if (registerOperation == 0)
	{
		viCheckErr (viPrintf (instrSession, "STAT:PRES\n"));
	}
	else
	{
		viCheckErr (viPrintf (instrSession, "STAT:%s:ENAB %ld\n", regOperArr[registerOperation], enable));
	}
	
    viCheckErr (rssfd_CheckStatus (instrSession)); 	
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function contains the commands for the querying status reporting 
/// HIFN  system.
/// HIFN     
/// HIFN     Note:
/// HIFN     
/// HIFN     For detailed description of Status Reporting System see also operating 
/// HIFN  manual.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     STATus:QUEStionable[:EVENt]?
/// HIFN     STATus:QUEStionable:CONDition?
/// HIFN     STATus:OPERation[:EVENt]?
/// HIFN     STATus:OPERation:CONDition?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RSSFD_init or 
/// HIPAR instrSession/RSSFD_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR statusRegisterQuery/This control selects the registers that will be queried. 
/// HIPAR registerPart/Selects register part to be queried.
/// HIPAR registerValue/Returns content of selected register.
ViStatus _VI_FUNC rssfd_GetStatusRegister(
	ViSession	instrSession,
	ViInt32		statusRegisterQuery,
	ViInt32		registerPart,
	ViInt32*	registerValue
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 

    viCheckParm (rssfd_invalidViInt32Range (statusRegisterQuery, RSSFD_VAL_REG_QUEST, RSSFD_VAL_REG_OPER), 2, "Status Register Query");
    
	viCheckParm (rssfd_invalidViInt32Range (registerPart, RSSFD_VAL_REG_COND, RSSFD_VAL_REG_EVENT), 3, "Register Part"); 	
	
    viCheckErr (viQueryf (instrSession, "STAT:%s:%s?\n", "%d", regOperArr[statusRegisterQuery], regPartArr[registerPart], registerValue));
	
    checkErr( rssfd_CheckStatus (instrSession)); 
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: rssfd_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/
/* ViInt32 */
ViStatus _VI_FUNC rssfd_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rssfd_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rssfd_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rssfd_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rssfd_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*****************************************************************************
 * Function: rssfd_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rssfd_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/* ViReal64 */
ViStatus _VI_FUNC rssfd_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/* ViString */
ViStatus _VI_FUNC rssfd_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rssfd_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/* ViSession */
ViStatus _VI_FUNC rssfd_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}


/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rssfd_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rssfd_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize >= RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE - 1 : bufferSize - 1] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: rssfd_CheckAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to check
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the rssfd_ATTR_RANGE_CHECK
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32,
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rssfd_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rssfd_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rssfd_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rssfd_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rssfd_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*****************************************************************************
 * Function: Reset
 * Purpose:  This function resets the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssfd_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*RST\n"));
        }

    checkErr( rssfd_DefaultInstrSetup (instrSession));
    checkErr( rssfd_CheckStatus (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);

    return error;
}

/*****************************************************************************
 * Function: Revision Query
 * Purpose:  This function returns the driver and instrument revisions.
 *****************************************************************************/
ViStatus _VI_FUNC rssfd_revision_query (ViSession instrSession, ViChar driverRev[],
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSSFD_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSSFD_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rssfd_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Error Query
 * Purpose:  This function queries the instrument error queue and returns
 *           the result.
 *****************************************************************************/
ViStatus _VI_FUNC rssfd_error_query (ViSession instrSession, ViInt32 *errCode,
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSSFD_IO_BUFFER_SIZE] = "";
    ViInt32     oldCode = 0;
    ViChar      oldMessage[256] = "";
    ViChar      *p2buf = NULL;
    ViInt32     i = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
    {
        p2buf = (char *)malloc (1);

        *p2buf='\0';
            
        do 
        {
            viCheckErr (rssfd_GetAttributeViString( instrSession, "", RSSFD_ATTR_SYSTEM_ERROR, RSSFD_IO_BUFFER_SIZE, buffer));
            if  (sscanf (buffer, "%ld,%*[^0-9a-zA-Z]%256[^\"]", &oldCode, oldMessage) != 2)
            {
                viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);
            }
            else
            {
                if (oldCode != 0)
                {
                    *errCode = oldCode;
                    
                    p2buf = (char *)realloc (p2buf, strlen(p2buf) + strlen (oldMessage) + 2);
        
                    strcat (p2buf, oldMessage);
        
                    strcat (p2buf, "\n");
                    
                    i++;
                }
                else if (i == 0)
                {
                    *errCode = oldCode;
    
                    strncpy (errMessage, oldMessage, 1024);
                }
            }

        } while (oldCode != 0);
        
        if (i > 0)
        {
            p2buf[strlen (p2buf) - 1] = '\0'; 
            
            strncpy (errMessage, p2buf, 1024);
        }
    }
    
    else
    {
        /* Simulate Error Query */
        *errCode = 0;
        strcpy (errMessage, "No error.");
    }

Error:
    if (p2buf) free (p2buf);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Error Message
 * Purpose:  This function translates the error codes returned by this
 *           instrument driver into user-readable strings.
 *
 *           Note:  The caller can pass VI_NULL for the vi parameter.  This
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC rssfd_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
            RSSFD_ERROR_CODES_AND_MSGS,
            {VI_NULL, VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Preset
 * Purpose:  This function initiates an instrument reset.
 *****************************************************************************/
ViStatus _VI_FUNC rssfd_Preset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rssfd_SetAttributeViString (instrSession, "", 
                RSSFD_ATTR_SYSTEM_PRESET, VI_NULL));
    
    checkErr( rssfd_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rssfd_init or 
/// HIPAR instrSession/rssfd_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rssfd_ClearStatus (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	checkErr (rssfd_WriteInstrData (instrSession, "*CLS\n"));
	
	checkErr( rssfd_CheckStatus (instrSession));

Error:
	(void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rssfd_init or 
/// HIPAR instrSession/rssfd_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
ViStatus _VI_FUNC rssfd_IDQueryResponse (ViSession instrSession,
                                             ViChar IDQueryResponse[])
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	checkErr (rssfd_QueryViString (instrSession, "*IDN?", 256, IDQueryResponse));

Error:
	(void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rssfd_init or 
/// HIPAR instrSession/rssfd_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rssfd_ProcessAllPreviousCommands (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	checkErr (rssfd_WriteInstrData (instrSession, "*WAI\n"));
	
Error:
	(void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rssfd_init or 
/// HIPAR instrSession/rssfd_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC rssfd_QueryOPC (ViSession instrSession, ViInt32 *OPC)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	checkErr (rssfd_QueryViInt32 (instrSession, "*OPC?", OPC));

Error:
	(void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Get Error and Clear Error Functions
 * Purpose:  These functions enable the instrument driver user to
 *           get or clear the error information the driver associates with the
 *           RS session.
 *****************************************************************************/
ViStatus _VI_FUNC rssfd_GetError (ViSession instrSession,
                                     ViStatus *errorCode,
                                     ViInt32 bufferSize,
                                     ViChar description[])
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE + 256]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
        {
        if (errorCode == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
            }
        checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }

    else
        {
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
        }

    if (description == VI_NULL)
        {
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
        }

    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rssfd_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rssfd_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }

    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) (strlen(errorMessage) + 1);

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }

    if (errorCode)
        {
        *errorCode = primary;
        }

Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

ViStatus _VI_FUNC rssfd_ClearError (ViSession instrSession)
{
    return Rs_ClearErrorInfo (instrSession);
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions
 * Purpose:  These functions enable the instrument driver user to
 *           write and read commands directly to and from the instrument.
 *
 *           Note:  These functions bypass the RS attribute state caching.
 *                  WriteInstrData invalidates the cached values for all
 *                  attributes.
 *****************************************************************************/

ViStatus _VI_FUNC rssfd_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    return Rs_WriteInstrData (instrSession, writeBuffer);
}

ViStatus _VI_FUNC rssfd_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/*****************************************************************************
 * Function: Query ViBoolean
 * Purpose:  This function queries the ViBoolean value.
 *****************************************************************************/
ViStatus _VI_FUNC rssfd_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%d", command, value));
    checkErr( rssfd_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function is used to read data from the instrument and write 
/// HIFN it to a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument
/// HIPAR source/to control computer.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from instrument to control computer applies.
 ViStatus _VI_FUNC rssfd_ReadToFileFromInstrument (ViSession instrSession,
                                                     ViString source,
                                                     ViString destination)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     count = 0;
    ViUInt32    ret_count = 0;
    ViUInt32    length = 0;
    ViChar      buffer[RSSFD_IO_BUFFER_SIZE] = "";
    ViUInt16 intfc = VI_INTF_GPIB;     
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
	viCheckParm (!source, 2, "Source");
	viCheckParm (!destination, 3, "Destination");    
    
	checkErr (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));
    switch (intfc)
    {
        case VI_INTF_ASRL: /* Serial intfc. */
            viCheckErr( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
        break;
    }
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
    
    viCheckErr (viPrintf(instrSession, ":MMEMory:DATA? '%s'\n", source));
    //Read header       
    viCheckErr (viRead(instrSession, (ViPBuf)buffer, 2,  &ret_count));
    if (sscanf (buffer, "#%1ld", &count) != 1)
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE); 
    viCheckErr (viRead (instrSession, (ViPBuf)buffer, (ViUInt32) count, &ret_count));
    if (sscanf (buffer, "%lu", &length)  != 1)
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);     
    viCheckErr(Rs_ReadToFile(instrSession, destination, (ViInt32) length, RS_VAL_TRUNCATE, &count)); 
    //Read \n and \r
    viCheckErr (viRead (instrSession, (ViPBuf)buffer, 2, &ret_count)); 
    if (count!=(ViInt32) length)
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);         
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);        
    checkErr( rssfd_CheckStatus (instrSession));
  
Error:
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
    switch (intfc)
    {
        case VI_INTF_ASRL:
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
        break;    
    }  
    (void) Rs_UnlockSession(instrSession, NULL);
    
    return error;
}

/// HIFN This function is used to read data from the host computer and 
/// HIFN write it to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control
/// HIPAR source/computer to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from control computer to instrument applies.
 ViStatus _VI_FUNC rssfd_WriteFromFileToInstrument (ViSession instrSession,
                                                      ViString source,
                                                      ViString destination)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     count = 0;
    ViUInt32    digits = 0;
    FILE        *sourceFile = NULL;
    ViUInt32    fileSize            = 0;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    viCheckParm (!source, 2, "Source");
    viCheckParm (!destination, 3, "Destination");  
        
    /* Open source file from host controller */
    if ((sourceFile = fopen (source, "rb")) == NULL)
        viCheckErr(RS_ERROR_CANNOT_OPEN_FILE);

    /* Get size of file */
    (void) fseek (sourceFile, 0, SEEK_END);
    fileSize = (ViUInt32) ftell (sourceFile);
    (void) fseek (sourceFile, 0, SEEK_SET);

    /* Create command header */
    digits = (ViUInt32) ((int)floor(log10((double)fileSize))) + 1;
    viCheckErr (viPrintf (instrSession, ":MMEM:DATA '%s',#%ld%ld", destination, digits, fileSize));
    
    viCheckErr (Rs_WriteFromFile(instrSession, source, (ViInt32) fileSize, 0, &count));
    if (count!=(ViInt32) fileSize)
        viCheckErr (RS_ERROR_WRITING_FILE);     
        
    checkErr( rssfd_CheckStatus (instrSession));
 
 Error:
    if (sourceFile) (void) fclose (sourceFile);
    
    (void) Rs_UnlockSession(instrSession, NULL);
    
    return error;
}
/*****************************************************************************
 * Function: Query ViInt32
 * Purpose:  This function queries the ViInt32 value.
 *****************************************************************************/
ViStatus _VI_FUNC rssfd_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rssfd_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Query ViReal64
 * Purpose:  This function queries the ViReal64 value.
 *****************************************************************************/
ViStatus _VI_FUNC rssfd_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rssfd_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 * Function: Query ViString
 * Purpose:  This function queries the ViString value.
 *****************************************************************************/
ViStatus _VI_FUNC rssfd_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rssfd_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: rssfd_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC rssfd_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rssfd_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}


/*****************************************************************************
 * Function: rssfd_RsClose
 * Purpose:  This function performs all of the drivers clean-up operations
 *           except for closing the RS session.  This function is called by
 *           rssfd_close or by an RS class driver.
 *
 *           Note:  This function must close the I/O session and set
 *           RS_ATTR_IO_SESSION to 0.
 *****************************************************************************/
ViStatus _VI_FUNC rssfd_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViSession   io      = VI_NULL;

    /* Do not lock here. The caller manages the lock. */

Error:

    if(io != 0)
        {
        ViSession   rmSession = VI_NULL;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession));
        
        viWrite (instrSession, (ViBuf)"&GTL\n", 5, NULL);
        
        (void) viClose (io);
        (void) viClose (rmSession);
        }
    
    return error;
}
